@@ -17,8 +17,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 //BEGIN Includes
@@ -70,7 +70,7 @@
 #include <kprocio.h>
 #include <kregexpeditorinterface.h>
 #include <krun.h>
-#include <kspell.h>
+#include <kseparator.h>
 #include <kstandarddirs.h>
 #include <ktempfile.h>
 
@@ -109,7 +109,7 @@
 #include <qwidgetstack.h>
 
 // trailing slash is important
-#define HLDOWNLOADPATH "http://www.kde.org/apps/kate/syntax/"
+#define HLDOWNLOADPATH "http://kate.kde.org/syntax/"
 
 //END
 
@@ -132,28 +132,6 @@ void KateConfigPage::somethingHasChanged ()
 }
 //END KateConfigPage
 
-//BEGIN KateSpellConfigPage
-KateSpellConfigPage::KateSpellConfigPage( QWidget* parent )
-  : KateConfigPage( parent)
-{
-  QVBoxLayout* l = new QVBoxLayout( this );
-  cPage = new KSpellConfig( this, 0L, 0L, false );
-  l->addWidget( cPage );
-  connect( cPage, SIGNAL( configChanged() ), this, SLOT( slotChanged() ) );
-}
-
-void KateSpellConfigPage::apply ()
-{
-  // nothing changed, no need to apply stuff
-  if (!changed())
-    return;
-
-  m_changed = false;
-  // kspell
-  cPage->writeGlobalSettings ();
-}
-//END KateSpellConfigPage
-
 //BEGIN KateIndentConfigTab
 const int KateIndentConfigTab::flags[] = {
     KateDocument::cfSpaceIndent,
@@ -174,10 +152,12 @@ KateIndentConfigTab::KateIndentConfigTab(QWidget *parent)
   QVGroupBox *gbAuto = new QVGroupBox(i18n("Automatic Indentation"), this);
 
   QHBox *indentLayout = new QHBox(gbAuto);
+  indentLayout->setSpacing(KDialog::spacingHint());
   QLabel *indentLabel = new QLabel(i18n("&Indentation mode:"), indentLayout);
   m_indentMode = new KComboBox (indentLayout);
   m_indentMode->insertStringList (KateAutoIndent::listModes());
   indentLabel->setBuddy(m_indentMode);
+  m_configPage = new QPushButton(SmallIconSet("configure"), i18n("Configure..."), indentLayout);
 
   opt[5] = new QCheckBox(i18n("Insert leading Doxygen \"*\" when typing"), gbAuto);
 
@@ -240,6 +220,10 @@ KateIndentConfigTab::KateIndentConfigTab(QWidget *parent)
       "Use a mix of tab and space characters for indentation.") );
   QWhatsThis::add(indentationWidth, i18n("The number of spaces to indent with."));
 
+  QWhatsThis::add(m_configPage, i18n(
+        "If this button is enabled, additional indenter specific options are "
+        "available and can be configured in an extra dialog.") );
+
   reload ();
 
   //
@@ -264,6 +248,8 @@ KateIndentConfigTab::KateIndentConfigTab(QWidget *parent)
   connect(rb1, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
   connect(rb2, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
   connect(rb3, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+
+  connect(m_configPage, SIGNAL(clicked()), this, SLOT(configPage()));
 }
 
 void KateIndentConfigTab::somethingToggled() {
@@ -277,6 +263,34 @@ void KateIndentConfigTab::indenterSelected (int index)
     opt[5]->setEnabled(true);
   else
     opt[5]->setEnabled(false);
+
+  m_configPage->setEnabled( KateAutoIndent::hasConfigPage(index) );
+}
+
+void KateIndentConfigTab::configPage()
+{
+  uint index = m_indentMode->currentItem();
+  if ( KateAutoIndent::hasConfigPage(index) )
+  {
+    KDialogBase dlg(this, "indenter_config_dialog", true, i18n("Configure Indenter"),
+      KDialogBase::Ok|KDialogBase::Cancel, KDialogBase::Cancel, true);
+
+    QVBox *box = new QVBox(&dlg);
+    box->setSpacing( KDialog::spacingHint() );
+    dlg.setMainWidget(box);
+    new QLabel("<qt><b>" + KateAutoIndent::modeDescription(index) + "</b></qt>", box);
+    new KSeparator(KSeparator::HLine, box);
+
+    IndenterConfigPage* page = KateAutoIndent::configPage(box, index);
+
+    if (!page) return;
+    box->setStretchFactor(page, 1);
+
+    connect( &dlg, SIGNAL(okClicked()), page, SLOT(apply()) );
+
+    dlg.resize(400, 300);
+    dlg.exec();
+  }
 }
 
 void KateIndentConfigTab::apply ()
@@ -324,11 +338,36 @@ void KateIndentConfigTab::reload ()
 //END KateIndentConfigTab
 
 //BEGIN KateSelectConfigTab
+const int KateSelectConfigTab::flags[] = {};
+
 KateSelectConfigTab::KateSelectConfigTab(QWidget *parent)
   : KateConfigPage(parent)
 {
+  int configFlags = KateDocumentConfig::global()->configFlags();
+
   QVBoxLayout *layout = new QVBoxLayout(this, 0, KDialog::spacingHint() );
 
+  QVGroupBox *gbCursor = new QVGroupBox(i18n("Text Cursor Movement"), this);
+
+  opt[0] = new QCheckBox(i18n("Smart ho&me"), gbCursor);
+  opt[0]->setChecked(configFlags & KateDocumentConfig::cfSmartHome);
+  connect(opt[0], SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+
+  opt[1] = new QCheckBox(i18n("Wrap c&ursor"), gbCursor);
+  opt[1]->setChecked(configFlags & KateDocumentConfig::cfWrapCursor);
+  connect(opt[1], SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+
+  e6 = new QCheckBox(i18n("&PageUp/PageDown moves cursor"), gbCursor);
+  e6->setChecked(KateDocumentConfig::global()->pageUpDownMovesCursor());
+  connect(e6, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+
+  e4 = new KIntNumInput(KateViewConfig::global()->autoCenterLines(), gbCursor);
+  e4->setRange(0, 1000000, 1, false);
+  e4->setLabel(i18n("Autocenter cursor (lines):"), AlignVCenter);
+  connect(e4, SIGNAL(valueChanged(int)), this, SLOT(slotChanged()));
+
+  layout->addWidget(gbCursor);
+
   QRadioButton *rb1, *rb2;
 
   m_tabs = new QButtonGroup( 1, Qt::Horizontal, i18n("Selection Mode"), this );
@@ -338,7 +377,6 @@ KateSelectConfigTab::KateSelectConfigTab(QWidget *parent)
   m_tabs->insert( rb1=new QRadioButton( i18n("&Normal"), m_tabs ), 0 );
   m_tabs->insert( rb2=new QRadioButton( i18n("&Persistent"), m_tabs ), 1 );
 
-
   layout->addStretch();
 
   QWhatsThis::add(rb1, i18n(
@@ -347,6 +385,24 @@ KateSelectConfigTab::KateSelectConfigTab(QWidget *parent)
   QWhatsThis::add(rb2, i18n(
         "Selections will stay even after cursor movement and typing."));
 
+  QWhatsThis::add(e4, i18n(
+        "Sets the number of lines to maintain visible above and below the "
+        "cursor when possible."));
+
+  QWhatsThis::add(opt[0], i18n(
+        "When selected, pressing the home key will cause the cursor to skip "
+        "whitespace and go to the start of a line's text."));
+
+    QWhatsThis::add(opt[1], i18n(
+        "When on, moving the insertion cursor using the <b>Left</b> and "
+        "<b>Right</b> keys will go on to previous/next line at beginning/end of "
+        "the line, similar to most editors.<p>When off, the insertion cursor "
+        "cannot be moved left of the line start, but it can be moved off the "
+        "line end, which can be very handy for programmers."));
+
+  QWhatsThis::add(e6, i18n("Selects whether the PageUp and PageDown keys should alter the vertical position of the cursor relative to the top of the view."));
+
+
   reload ();
 
   //
@@ -364,23 +420,31 @@ void KateSelectConfigTab::apply ()
     return;
   m_changed = false;
 
+  KateViewConfig::global()->configStart ();
   KateDocumentConfig::global()->configStart ();
 
   int configFlags = KateDocumentConfig::global()->configFlags();
 
-  configFlags &= ~KateDocumentConfig::cfPersistent; // clear persistent
+  configFlags &= ~KateDocumentConfig::cfSmartHome;
+  configFlags &= ~KateDocumentConfig::cfWrapCursor;
 
-  if (m_tabs->id (m_tabs->selected()) == 1)
-    configFlags |= KateDocumentConfig::cfPersistent; // set flag if checked
+  if (opt[0]->isChecked()) configFlags |= KateDocumentConfig::cfSmartHome;
+  if (opt[1]->isChecked()) configFlags |= KateDocumentConfig::cfWrapCursor;
 
   KateDocumentConfig::global()->setConfigFlags(configFlags);
 
+  KateViewConfig::global()->setAutoCenterLines(QMAX(0, e4->value()));
+  KateDocumentConfig::global()->setPageUpDownMovesCursor(e6->isChecked());
+
+  KateViewConfig::global()->setPersistentSelection (m_tabs->id (m_tabs->selected()) == 1);
+
   KateDocumentConfig::global()->configEnd ();
+  KateViewConfig::global()->configEnd ();
 }
 
 void KateSelectConfigTab::reload ()
 {
-  if (KateDocumentConfig::global()->configFlags() & KateDocumentConfig::cfPersistent)
+  if (KateViewConfig::global()->persistentSelection())
     m_tabs->setButton (1);
   else
     m_tabs->setButton (0);
@@ -389,8 +453,8 @@ void KateSelectConfigTab::reload ()
 
 //BEGIN KateEditConfigTab
 const int KateEditConfigTab::flags[] = {KateDocument::cfWordWrap,
-  KateDocument::cfAutoBrackets, KateDocument::cfShowTabs, KateDocument::cfSmartHome,
-  KateDocument::cfWrapCursor, KateDocumentConfig::cfReplaceTabsDyn, KateDocumentConfig::cfRemoveTrailingDyn};
+  KateDocument::cfAutoBrackets, KateDocument::cfShowTabs,
+  KateDocumentConfig::cfReplaceTabsDyn, KateDocumentConfig::cfRemoveTrailingDyn};
 
 KateEditConfigTab::KateEditConfigTab(QWidget *parent)
   : KateConfigPage(parent)
@@ -400,14 +464,14 @@ KateEditConfigTab::KateEditConfigTab(QWidget *parent)
 
   QVGroupBox *gbWhiteSpace = new QVGroupBox(i18n("Tabulators"), this);
 
-  opt[2] = new QCheckBox(i18n("&Show tabs"), gbWhiteSpace);
+  opt[3] = new QCheckBox( i18n("&Insert spaces instead of tabulators"), gbWhiteSpace );
+  opt[3]->setChecked( configFlags & KateDocumentConfig::cfReplaceTabsDyn );
+  connect( opt[3], SIGNAL(toggled(bool)), this, SLOT(slotChanged()) );
+
+  opt[2] = new QCheckBox(i18n("&Show tabulators"), gbWhiteSpace);
   opt[2]->setChecked(configFlags & flags[2]);
   connect(opt[2], SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
 
-  opt[5] = new QCheckBox( i18n("&Replace tabs with spaces"), gbWhiteSpace );
-  opt[5]->setChecked( configFlags & KateDocumentConfig::cfReplaceTabsDyn );
-  connect( opt[5], SIGNAL(toggled(bool)), this, SLOT(slotChanged()) );
-
   e2 = new KIntNumInput(KateDocumentConfig::global()->tabWidth(), gbWhiteSpace);
   e2->setRange(1, 16, 1, false);
   e2->setLabel(i18n("Tab width:"), AlignVCenter);
@@ -421,6 +485,10 @@ KateEditConfigTab::KateEditConfigTab(QWidget *parent)
   opt[0]->setChecked(KateDocumentConfig::global()->wordWrap());
   connect(opt[0], SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
 
+  m_wwmarker = new QCheckBox( i18n("&Show static word wrap marker (if applicable)"), gbWordWrap );
+  m_wwmarker->setChecked( KateRendererConfig::global()->wordWrapMarker() );
+  connect(m_wwmarker, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+
   e1 = new KIntNumInput(KateDocumentConfig::global()->wordWrapAt(), gbWordWrap);
   e1->setRange(20, 200, 1, false);
   e1->setLabel(i18n("Wrap words at:"), AlignVCenter);
@@ -428,31 +496,10 @@ KateEditConfigTab::KateEditConfigTab(QWidget *parent)
 
   mainLayout->addWidget(gbWordWrap);
 
-  QVGroupBox *gbCursor = new QVGroupBox(i18n("Text Cursor Movement"), this);
-
-  opt[3] = new QCheckBox(i18n("Smart ho&me"), gbCursor);
-  opt[3]->setChecked(configFlags & flags[3]);
-  connect(opt[3], SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
-
-  opt[4] = new QCheckBox(i18n("Wrap c&ursor"), gbCursor);
-  opt[4]->setChecked(configFlags & flags[4]);
-  connect(opt[4], SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
-
-  e6 = new QCheckBox(i18n("&PageUp/PageDown moves cursor"), gbCursor);
-  e6->setChecked(KateDocumentConfig::global()->pageUpDownMovesCursor());
-  connect(e6, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
-
-  e4 = new KIntNumInput(KateViewConfig::global()->autoCenterLines(), gbCursor);
-  e4->setRange(0, 1000000, 1, false);
-  e4->setLabel(i18n("Autocenter cursor (lines):"), AlignVCenter);
-  connect(e4, SIGNAL(valueChanged(int)), this, SLOT(slotChanged()));
-
-  mainLayout->addWidget(gbCursor);
-
-  opt[6] = new QCheckBox( i18n("Remove &trailing spaces"), this );
-  mainLayout->addWidget( opt[6] );
-  opt[6]->setChecked( configFlags & KateDocumentConfig::cfRemoveTrailingDyn );
-  connect( opt[6], SIGNAL(toggled(bool)), this, SLOT(slotChanged()) );
+  opt[4] = new QCheckBox( i18n("Remove &trailing spaces"), this );
+  mainLayout->addWidget( opt[4] );
+  opt[4]->setChecked( configFlags & KateDocumentConfig::cfRemoveTrailingDyn );
+  connect( opt[4], SIGNAL(toggled(bool)), this, SLOT(slotChanged()) );
 
   opt[1] = new QCheckBox(i18n("Auto &brackets"), this);
   mainLayout->addWidget(opt[1]);
@@ -500,21 +547,10 @@ KateEditConfigTab::KateEditConfigTab(QWidget *parent)
   QWhatsThis::add(opt[2], i18n(
         "The editor will display a symbol to indicate the presence of a tab in "
         "the text."));
-  QWhatsThis::add(opt[3], i18n(
-        "When selected, pressing the home key will cause the cursor to skip "
-        "whitespace and go to the start of a line's text."));
+
   QWhatsThis::add(e3, i18n(
         "Sets the number of undo/redo steps to record. More steps uses more memory."));
-  QWhatsThis::add(e4, i18n(
-        "Sets the number of lines to maintain visible above and below the "
-        "cursor when possible."));
-  QWhatsThis::add(opt[4], i18n(
-        "When on, moving the insertion cursor using the <b>Left</b> and "
-        "<b>Right</b> keys will go on to previous/next line at beginning/end of "
-        "the line, similar to most editors.<p>When off, the insertion cursor "
-        "cannot be moved left of the line start, but it can be moved off the "
-        "line end, which can be very handy for programmers."));
-  QWhatsThis::add(e6, i18n("Selects whether the PageUp and PageDown keys should alter the vertical position of the cursor relative to the top of the view."));
+
   QString gstfwt = i18n(
         "This determines where KateView will get the search text from "
         "(this will be automatically entered into the Find Text dialog): "
@@ -540,13 +576,18 @@ KateEditConfigTab::KateEditConfigTab(QWidget *parent)
         "will fall back to the last search text.");
   QWhatsThis::add(e5Label, gstfwt);
   QWhatsThis::add(e5, gstfwt);
-  QWhatsThis::add( opt[5], i18n(
+  QWhatsThis::add( opt[3], i18n(
       "If this is enabled, the editor will calculate the number of spaces up to "
       "the next tab position as defined by the tab width, and insert that number "
       "of spaces instead of a TAB character." ) );
-  QWhatsThis::add( opt[6], i18n(
+  QWhatsThis::add( opt[4], i18n(
       "If this is enabled, the editor will remove any trailing whitespace on "
       "lines when they are left by the insertion cursor.") );
+  QWhatsThis::add( m_wwmarker, i18n(
+        "<p>If this option is checked, a vertical line will be drawn at the word "
+        "wrap column as defined in the <strong>Editing</strong> properties."
+        "<p>Note that the word wrap marker is only drawn if you use a fixed "
+        "pitch font." ));
 }
 
 void KateEditConfigTab::apply ()
@@ -577,9 +618,9 @@ void KateEditConfigTab::apply ()
   else
     KateDocumentConfig::global()->setUndoSteps(e3->value());
 
-  KateViewConfig::global()->setAutoCenterLines(QMAX(0, e4->value()));
   KateViewConfig::global()->setTextToSearchMode(e5->currentItem());
-  KateDocumentConfig::global()->setPageUpDownMovesCursor(e6->isChecked());
+
+  KateRendererConfig::global()->setWordWrapMarker (m_wwmarker->isChecked());
 
   KateDocumentConfig::global()->configEnd ();
   KateViewConfig::global()->configEnd ();
@@ -587,7 +628,6 @@ void KateEditConfigTab::apply ()
 
 void KateEditConfigTab::reload ()
 {
-
 }
 //END KateEditConfigTab
 
@@ -619,8 +659,6 @@ KateViewDefaultsConfig::KateViewDefaultsConfig(QWidget *parent)
   m_dynwrapAlignLevel->setSuffix(i18n("% of View Width"));
   m_dynwrapAlignLevel->setSpecialValueText(i18n("Disabled"));
 
-  m_wwmarker = new QCheckBox( i18n("&Show static word wrap marker (if applicable)"), gbWordWrap );
-
   blay->addWidget(gbWordWrap);
 
   QVGroupBox *gbFold = new QVGroupBox(i18n("Code Folding"), this);
@@ -645,6 +683,11 @@ KateViewDefaultsConfig::KateViewDefaultsConfig(QWidget *parent)
   m_bmSort->insert( rb2=new QRadioButton( i18n("By c&reation"), m_bmSort ), 1 );
 
   blay->addWidget(m_bmSort, 0 );
+
+  m_showIndentLines = new QCheckBox(i18n("Show indentation lines"), this);
+  m_showIndentLines->setChecked(KateRendererConfig::global()->showIndentationLines());
+  blay->addWidget(m_showIndentLines);
+
   blay->addStretch(1000);
 
   QWhatsThis::add(m_dynwrap,i18n(
@@ -663,11 +706,6 @@ KateViewDefaultsConfig::KateViewDefaultsConfig(QWidget *parent)
         "example, at 50%, lines whose indentation levels are deeper than 50% of "
         "the width of the screen will not have vertical alignment applied to "
         "subsequent wrapped lines.</p>"));
-  QWhatsThis::add( m_wwmarker, i18n(
-        "<p>If this option is checked, a vertical line will be drawn at the word "
-        "wrap column as defined in the <strong>Editing</strong> properties."
-        "<p>Note that the word wrap marker is only drawn if you use a fixed "
-        "pitch font." ));
   QWhatsThis::add(m_line,i18n(
         "If this option is checked, every new view will display line numbers "
         "on the left hand side."));
@@ -689,6 +727,9 @@ KateViewDefaultsConfig::KateViewDefaultsConfig(QWidget *parent)
   QWhatsThis::add(rb2,i18n(
         "Each new bookmark will be added to the bottom, independently from "
         "where it is placed in the document."));
+  QWhatsThis::add(m_showIndentLines, i18n(
+        "If this is enabled, the editor will display vertical line to help "
+        "identifying indent lines.") );
 
   reload();
 
@@ -699,7 +740,6 @@ KateViewDefaultsConfig::KateViewDefaultsConfig(QWidget *parent)
   connect(m_dynwrap, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
   connect(m_dynwrapIndicatorsCombo, SIGNAL(activated(int)), this, SLOT(slotChanged()));
   connect(m_dynwrapAlignLevel, SIGNAL(valueChanged(int)), this, SLOT(slotChanged()));
-  connect(m_wwmarker, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
   connect(m_icons, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
   connect(m_scrollBarMarks, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
   connect(m_line, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
@@ -707,6 +747,7 @@ KateViewDefaultsConfig::KateViewDefaultsConfig(QWidget *parent)
   connect(m_collapseTopLevel, SIGNAL(toggled(bool)), this, SLOT(slotChanged()) );
   connect(rb1, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
   connect(rb2, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(m_showIndentLines, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
 }
 
 KateViewDefaultsConfig::~KateViewDefaultsConfig()
@@ -726,13 +767,14 @@ void KateViewDefaultsConfig::apply ()
   KateViewConfig::global()->setDynWordWrap (m_dynwrap->isChecked());
   KateViewConfig::global()->setDynWordWrapIndicators (m_dynwrapIndicatorsCombo->currentItem ());
   KateViewConfig::global()->setDynWordWrapAlignIndent(m_dynwrapAlignLevel->value());
-  KateRendererConfig::global()->setWordWrapMarker (m_wwmarker->isChecked());
   KateViewConfig::global()->setLineNumbers (m_line->isChecked());
   KateViewConfig::global()->setIconBar (m_icons->isChecked());
   KateViewConfig::global()->setScrollBarMarks (m_scrollBarMarks->isChecked());
   KateViewConfig::global()->setFoldingBar (m_folding->isChecked());
   KateViewConfig::global()->setBookmarkSort (m_bmSort->id (m_bmSort->selected()));
 
+  KateRendererConfig::global()->setShowIndentationLines(m_showIndentLines->isChecked());
+
   KateRendererConfig::global()->configEnd ();
   KateViewConfig::global()->configEnd ();
 }
@@ -742,12 +784,12 @@ void KateViewDefaultsConfig::reload ()
   m_dynwrap->setChecked(KateViewConfig::global()->dynWordWrap());
   m_dynwrapIndicatorsCombo->setCurrentItem( KateViewConfig::global()->dynWordWrapIndicators() );
   m_dynwrapAlignLevel->setValue(KateViewConfig::global()->dynWordWrapAlignIndent());
-  m_wwmarker->setChecked( KateRendererConfig::global()->wordWrapMarker() );
   m_line->setChecked(KateViewConfig::global()->lineNumbers());
   m_icons->setChecked(KateViewConfig::global()->iconBar());
   m_scrollBarMarks->setChecked(KateViewConfig::global()->scrollBarMarks());
   m_folding->setChecked(KateViewConfig::global()->foldingBar());
-  m_bmSort->setButton( KateViewConfig::global()->bookmarkSort()  );
+  m_bmSort->setButton( KateViewConfig::global()->bookmarkSort() );
+  m_showIndentLines->setChecked(KateRendererConfig::global()->showIndentationLines());
 }
 
 void KateViewDefaultsConfig::reset () {;}
@@ -815,6 +857,8 @@ KateSaveConfigTab::KateSaveConfigTab( QWidget *parent )
   m_eol = new KComboBox (e5Layout);
   e5Label->setBuddy(m_eol);
 
+  allowEolDetection = new QCheckBox(i18n("&Automatic end of line detection"), gbEnc);
+
   m_eol->insertItem (i18n("UNIX"));
   m_eol->insertItem (i18n("DOS/Windows"));
   m_eol->insertItem (i18n("Macintosh"));
@@ -830,12 +874,9 @@ KateSaveConfigTab::KateSaveConfigTab( QWidget *parent )
   blockCount->setValue (KateBuffer::maxLoadedBlocks());
   blockCountLabel->setBuddy(blockCount);
 
-  QVGroupBox *gbWhiteSpace = new QVGroupBox(i18n("Automatic Cleanups on Save"), this);
+  QVGroupBox *gbWhiteSpace = new QVGroupBox(i18n("Automatic Cleanups on Load/Save"), this);
   layout->addWidget( gbWhiteSpace );
 
-  replaceTabs = new QCheckBox(i18n("Replace &tabs with spaces"), gbWhiteSpace);
-  replaceTabs->setChecked(configFlags & KateDocument::cfReplaceTabs);
-
   removeSpaces = new QCheckBox(i18n("Re&move trailing spaces"), gbWhiteSpace);
   removeSpaces->setChecked(configFlags & KateDocument::cfRemoveSpaces);
 
@@ -864,16 +905,16 @@ KateSaveConfigTab::KateSaveConfigTab( QWidget *parent )
 
   layout->addStretch();
 
-  QWhatsThis::add(replaceTabs, i18n(
-        "KateView will replace any tabs with the number of spaces indicated in "
-        "the Tab Width: entry."));
   QWhatsThis::add(removeSpaces, i18n(
-        "KateView will automatically eliminate extra spaces at the ends of "
-        "lines of text."));
+        "The editor will automatically eliminate extra spaces at the ends of "
+        "lines of text while loading/saving the file."));
   QWhatsThis::add( gb, i18n(
         "<p>Backing up on save will cause Kate to copy the disk file to "
         "'&lt;prefix&gt;&lt;filename&gt;&lt;suffix&gt;' before saving changes."
         "<p>The suffix defaults to <strong>~</strong> and prefix is empty by default" ) );
+  QWhatsThis::add( allowEolDetection, i18n(
+        "Check this if you want the editor to autodetect the end of line type."
+        "The first found end of line type will be used for the whole file.") );
   QWhatsThis::add( cbLocalFiles, i18n(
         "Check this if you want backups of local files when saving") );
   QWhatsThis::add( cbRemoteFiles, i18n(
@@ -901,8 +942,8 @@ KateSaveConfigTab::KateSaveConfigTab( QWidget *parent )
 
   connect(m_encoding, SIGNAL(activated(int)), this, SLOT(slotChanged()));
   connect(m_eol, SIGNAL(activated(int)), this, SLOT(slotChanged()));
+  connect( allowEolDetection, SIGNAL( toggled(bool) ), this, SLOT( slotChanged() ) );
   connect(blockCount, SIGNAL(valueChanged(int)), this, SLOT(slotChanged()));
-  connect(replaceTabs, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
   connect(removeSpaces, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
   connect( cbLocalFiles, SIGNAL( toggled(bool) ), this, SLOT( slotChanged() ) );
   connect( cbRemoteFiles, SIGNAL( toggled(bool) ), this, SLOT( slotChanged() ) );
@@ -945,9 +986,6 @@ void KateSaveConfigTab::apply()
 
   int configFlags = KateDocumentConfig::global()->configFlags();
 
-  configFlags &= ~KateDocument::cfReplaceTabs; // clear flag
-  if (replaceTabs->isChecked()) configFlags |= KateDocument::cfReplaceTabs; // set flag if checked
-
   configFlags &= ~KateDocument::cfRemoveSpaces; // clear flag
   if (removeSpaces->isChecked()) configFlags |= KateDocument::cfRemoveSpaces; // set flag if checked
 
@@ -956,6 +994,7 @@ void KateSaveConfigTab::apply()
   KateDocumentConfig::global()->setEncoding((m_encoding->currentItem() == 0) ? "" : KGlobal::charsets()->encodingForName(m_encoding->currentText()));
 
   KateDocumentConfig::global()->setEol(m_eol->currentItem());
+  KateDocumentConfig::global()->setAllowEolDetection(allowEolDetection->isChecked());
 
   KateDocumentConfig::global()->configEnd ();
 }
@@ -988,6 +1027,7 @@ void KateSaveConfigTab::reload()
 
   // eol
   m_eol->setCurrentItem(KateDocumentConfig::global()->eol());
+  allowEolDetection->setChecked(KateDocumentConfig::global()->allowEolDetection());
 
   dirSearchDepth->setValue(KateDocumentConfig::global()->searchDirConfigDepth());
 
