@@ -1,6 +1,5 @@
 /* This file is part of the KDE libraries
     Copyright (C) 1999 Lars Knoll (knoll@kde.org)
-    $Id: kcharsets.cpp,v 1.146.2.1 2004/10/29 16:42:36 goutte Exp $
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -14,11 +13,12 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 #include "kcharsets.h"
 
+#include "kqiodevicegzip_p.h"
 #include "kentities.c"
 
 #include <kapplication.h>
@@ -34,11 +34,11 @@
 #include <qtextcodec.h>
 #include <qmap.h>
 #include <qcstring.h>
+#include <qdir.h>
+#include <qregexp.h>
 
 #include <assert.h>
 
-#define CHARSETS_COUNT 33
-
 static const char * const language_names[] = {
 	I18N_NOOP( "Other" ),
 	I18N_NOOP( "Arabic" ),
@@ -56,7 +56,9 @@ static const char * const language_names[] = {
 	I18N_NOOP( "Western European" ),
 	I18N_NOOP( "Tamil" ),
 	I18N_NOOP( "Unicode" ),
-	I18N_NOOP( "Northern Saami" )
+	I18N_NOOP( "Northern Saami" ),
+        I18N_NOOP( "Vietnamese" ),
+        I18N_NOOP( "South-Eastern Europe" )
 };
 
 // this list gives the charsets that can be used to display a file given in a certain encoding.
@@ -82,7 +84,9 @@ static const char* const charsets_for_encoding[] = {
     "iso 8859-9",            "iso8859-9","unicode","iso8859-1", 0,
     "iso 8859-11",           "iso8859-11", 0,
     "iso 8859-13",           "iso8859-13","unicode","iso8859-4", "iso8859-1", 0,
+    "iso 8859-14",           "iso8859-15", "iso8859-1", 0,
     "iso 8859-15",           "iso8859-15","unicode","iso8859-1", 0,
+    "iso 8859-16",           "iso8859-2", 0, // ISO-8859-2 is not a replacement
     "utf8",                  "unicode","iso8859-1", 0,
     "utf16",                 "unicode","iso8859-1", 0,
     "iso-10646-ucs-2",       "unicode","iso8859-1", 0,
@@ -94,6 +98,7 @@ static const char* const charsets_for_encoding[] = {
     "cp 1255",               "iso8859-8", 0,
     "cp 1256",               "unicode","iso8859-6", 0,
     "cp 1257",               "iso8859-13", "iso8859-4", 0,
+    "cp 1258",               "iso8859-1", 0, // ISO-8859-1 is not a replacement
     "ibm850",                "ibm850","unicode","iso8859-1", 0,
     "ibm852",                "unicode","iso-8859-2", 0,
     "ibm866",                "ibm866","cp 1251","koi8-u","koi8-r","iso8859-5", 0,
@@ -102,13 +107,14 @@ static const char* const charsets_for_encoding[] = {
     "sjis",                  "eucjp","unicode","iso8859-1", 0,
     "jis7",                  "eucjp","unicode","iso8859-1", 0,
     "big5",                  "big5","unicode","iso8859-1", 0,
-    "gbk",                   "gb2312.1980-0","gbk-0","unicode","iso8859-1", 0,
+    "gbk",                   "gb2312.1980-0","gbk-0","unicode","iso8859-1", "gb_2312-80", 0,
     "gb18030",               "gb18030.2000-1", "gb18030.2000-0", "unicode", "gbk-0", "gb2313.1980-0", "iso8859-1", 0,
     "gb2312",                "gb2312.1980-0","unicode","iso8859-1", 0,
     "euckr",                 "euckr","unicode","iso8859-1", 0,
     "tscii",                 "tscii", 0,
-    "pt 154",                "pt 154","cp 1251","koi8-u","koi8-r","iso8859-5", 0,
+//    "pt 154",                "pt 154","cp 1251","koi8-u","koi8-r","iso8859-5", 0,
     "winsami2",              "winsami2", "cp1252", "unicode", 0,
+    "cp 874",                "tis620", 0,
     0 }; // extra 0 for end
 
 // 0 other
@@ -128,6 +134,9 @@ static const char* const charsets_for_encoding[] = {
 // 14 Tamil
 // 15 Unicode
 // 16 Northern Sami
+// 17 Vietnamese
+// 18 South-Eastern Europe
+// ### FIXME KDE4: the name of the encodings should mostly be uppercase
 static struct LanguageForEncoding
     {
     const char* index;
@@ -135,12 +144,14 @@ static struct LanguageForEncoding
     } const language_for_encoding[] = {
     { "iso 8859-1", 13 },
     { "iso 8859-15", 13 },
+    { "iso 8859-14", 13 },
     { "cp 1252", 13 },
     { "ibm850", 13 },
     { "iso 8859-2", 3 },
     { "iso 8859-3", 3 },
     { "iso 8859-4", 2 },
     { "iso 8859-13", 2 },
+    { "iso 8859-16", 18 },
     { "cp 1250", 3 },
     { "cp 1254", 12 },
     { "cp 1257", 2 },
@@ -149,7 +160,7 @@ static struct LanguageForEncoding
     { "iso 8859-5", 6 },
     { "cp 1251", 6 },
     { "koi8-u", 6 },
-    { "pt 154", 6 },
+//    { "pt 154", 6 },
     { "ibm866", 6 },
     { "big5", 5 },
     { "gb18030", 4 },
@@ -169,9 +180,12 @@ static struct LanguageForEncoding
     { "iso 8859-9", 12 },
     { "tis620", 11 },
     { "iso 8859-11", 11 },
+    { "cp 874", 11 },
+    { "cp 1258", 17 },
+    { "tscii", 14 },
     { "utf8", 15 },
     { "utf16", 15 },
-    { "utf7", 15 },
+    { "utf7", 15 }, // ### FIXME: UTF-7 is not in Qt
     { "ucs2", 15 },
     { "iso-10646-ucs-2", 15 },
     { "winsami2", 16},
@@ -184,27 +198,25 @@ static struct Builtin
     const char* data;
     } const builtin[] = {
     { "iso-ir-111", "koi8-r" },
-    { "koi8-ru", "koi8-u" },
-    { "koi8r", "koi8-r" },
-    { "koi8u", "koi8-u" },
-    { "koi unified", "koi8-r" },
+    { "koi8-ru", "koi8-u" }, // ### Qt 3.3 maps it to koi8-r
+    { "koi unified", "koi8-r" }, // ### FIXME: Qt 3.3 seems to map this to EUC-KR, so this mapping is too late :-(
+    // Using ISO-8859-1 for ASCII is an approximation at write
     { "us-ascii", "iso 8859-1" },
     { "usascii", "iso 8859-1" },
+    { "ascii", "iso 8859-1" },
     { "x-utf-8", "utf-8" },
-    { "x-utf-7", "utf-7" },
-    { "unicode-1-1-utf-7", "utf-7" },
+    { "x-utf-7", "utf-7" }, // ### FIXME: UTF-7 is not in Qt 
+    { "unicode-1-1-utf-7", "utf-7" }, // ### FIXME: UTF-7 is not in Qt
+    { "utf-16", "iso-10646-ucs-2" },
+    { "utf16", "iso-10646-ucs-2" },
     { "ucs2", "iso-10646-ucs-2" },
     { "iso10646-1", "iso-10646-ucs-2" },
     { "gb18030.2000-1", "gb18030" },
     { "gb18030.2000-0", "gb18030" },
     { "gbk-0", "gbk" },
-    { "gb2312", "gbk" },
     { "gb2312.1980-0", "gbk" },
-    { "big5-0", "big5" },
-    { "euc-kr", "euckr" },
+    { "gb_2312-80", "gbk" },/* this one is not official, but MS is using it :/ */
     { "x-euc-kr", "euckr" },
-    { "euc-jp", "eucjp" },
-    { "x-euc-jp", "eucjp" },
     { "jisx0201.1976-0", "eucjp" },
     { "jisx0208.1983-0", "eucjp" },
     { "jisx0208.1990-0", "eucjp" },
@@ -212,10 +224,6 @@ static struct Builtin
     { "jisx0212.1990-0", "eucjp" },
     { "jisx0213.2000-1", "eucjp" },
     { "jisx0213.2000-2", "eucjp" },
-    { "shift_jis", "sjis" },
-    { "shift-jis", "sjis" },
-    { "x-sjis", "sjis" },
-    { "iso-2022-jp", "jis7" },
     { "windows850", "ibm850" },
     { "windows866", "ibm866" },
     { "windows1251", "cp 1251" },
@@ -225,16 +233,9 @@ static struct Builtin
     { "windows1255", "cp 1255" },
     { "windows1256", "cp 1256" },
     { "windows1257", "cp 1257" },
+    { "windows1258", "cp 1258" },
     { "windows-850", "ibm850" },
     { "windows-866", "ibm866" },
-    { "windows-1250", "cp 1250" },
-    { "windows-1251", "cp 1251" },
-    { "windows-1252", "cp 1252" },
-    { "windows-1253", "cp 1253" },
-    { "windows-1254", "cp 1254" },
-    { "windows-1255", "cp 1255" },
-    { "windows-1256", "cp 1256" },
-    { "windows-1257", "cp 1257" },
     { "x-windows-850", "ibm850" },
     { "x-windows-866", "ibm866" },
     { "x-windows-1250", "cp 1250" },
@@ -245,8 +246,11 @@ static struct Builtin
     { "x-windows-1255", "cp 1255" },
     { "x-windows-1256", "cp 1256" },
     { "x-windows-1257", "cp 1257" },
+    { "x-windows-1258", "cp 1258" },
+    { "cp819", "iso 8859-1" },
     { "cp850", "ibm850" },
     { "cp866", "ibm866" },
+    { "cp-819", "iso 8859-1" },
     { "cp-850", "ibm850" },
     { "cp-866", "ibm866" },
     { "cp-1250", "cp 1250" },
@@ -257,6 +261,8 @@ static struct Builtin
     { "cp-1255", "cp 1255" },
     { "cp-1256", "cp 1256" },
     { "cp-1257", "cp 1257" },
+    { "cp-1258", "cp 1258" },
+    { "cp-10000", "apple roman" },
     { "x-cp-850", "ibm850" },
     { "x-cp-866", "ibm866" },
     { "x-cp-1250", "cp 1250" },
@@ -267,39 +273,23 @@ static struct Builtin
     { "x-cp-1255", "cp 1255" },
     { "x-cp-1256", "cp 1256" },
     { "x-cp-1257", "cp 1257" },
-    { "tis620", "iso 8859-11" },
-    { "tis-620", "iso 8859-11" },
+    { "x-cp-1258", "cp 1258" },
+    { "x-cp-10000", "apple roman" },
+    { "ibm819", "iso 8859-1" },
     { "thai-tis620", "iso 8859-11" },
-    { "windows-874", "iso 8859-11" },
-    { "windows874", "iso 8859-11" },
-    { "x-windows-874", "iso 8859-11" },
-    { "cp874", "iso 8859-11" },
-    { "cp-874", "iso 8859-11" },
-    { "x-cp-874", "iso 8859-11" },
+    { "windows-874", "cp 874" },
+    { "windows874", "cp 874" },
+    { "x-windows-874", "cp 874" },
+    { "x-cp-874", "cp 874" },
+    { "ibm 874", "cp 874" },
+    { "ibm874", "cp 874" }, // Qt4 name
+    { "x-ibm874", "cp 874" },
     { "ksc5601.1987-0", "euckr" },
-    { "ks_c_5601-1987", "euckr" },
-    { "iso-8859-1", "iso 8859-1" },
-    { "iso-8859-2", "iso 8859-2" },
-    { "iso-8859-3", "iso 8859-3" },
-    { "iso-8859-4", "iso 8859-4" },
-    { "iso-8859-5", "iso 8859-5" },
-    { "iso-8859-6", "iso 8859-6" },
-    { "iso-8859-7", "iso 8859-7" },
-    { "iso-8859-8", "iso 8859-8" },
-    { "iso-8859-9", "iso 8859-9" },
-    { "iso-8859-10", "iso 8859-10" },
-    { "iso-8859-11", "iso 8859-11" },
-    { "iso-8859-12", "iso 8859-12" },
-    { "iso-8859-13", "iso 8859-13" },
-    { "iso-8859-14", "iso 8859-14" },
-    { "iso-8859-15", "iso 8859-15" },
-    { "tscii", "tscii" },
-    { "paratype-154", "pt 154" },
-    { "pt-154", "pt 154" },
     { "x-winsami2", "winsami2" },
     { "x-mac-roman", "apple roman" },
     { "macintosh", "apple roman" },
     { "mac", "apple roman" },
+    { "csiso2022jp", "jis7" }, // See bug #77243
     { 0, 0 }};
 
 // some different names for the encodings defined in the charmaps files.
@@ -328,7 +318,10 @@ static struct ConversionHints
     { "cp1250", "iso-8859-2" },
     { "koi8-r", "iso-8859-5" },
     { "koi8-u", "koi8-r" },
-    { "utf16", "iso-10646-ucs-2" },
+    // KDE had always "CP 1251" as best fallback to PT 154. Now that Qt does not offer this encoding anymore, it is our fallback.
+    { "pt 154", "cp 1251" },
+    { "paratype-154", "cp 1251" },
+    { "pt-154", "cp 1251" },
     { 0, 0 }};
 
 
@@ -393,7 +386,7 @@ QChar KCharsets::fromEntity(const QString &str)
             QString tmp(str.unicode()+pos, str.length()-pos);
             res = tmp.toInt(&ok, 16);
         } else {
-            //  '&#0000', deciaml character reference
+            //  '&#0000', decimal character reference
             QString tmp(str.unicode()+pos, str.length()-pos);
             res = tmp.toInt(&ok, 10);
         }
@@ -471,7 +464,7 @@ QString KCharsets::resolveEntities( const QString &input )
 
         const uint ampersandPos = ampersand - text.unicode();
 
-        text[ ampersandPos ] = entityValue;
+        text[ (int)ampersandPos ] = entityValue;
         text.remove( ampersandPos + 1, entityLength + 1 );
         p = text.unicode() + ampersandPos;
         end = text.unicode() + text.length();
@@ -602,7 +595,6 @@ QTextCodec *KCharsets::codecForName(const QString &n, bool &ok) const
     {
     KConfigGroupSaver cfgsav( KGlobal::config(), "i18n" );
     dir = KGlobal::config()->readPathEntry("i18ndir", QString::fromLatin1("/usr/share/i18n/charmaps"));
-    dir += "/";
     }
 
     // these are codecs not included in Qt. They can be build up if the corresponding charmap
@@ -613,7 +605,62 @@ QTextCodec *KCharsets::codecForName(const QString &n, bool &ok) const
         cname = name;
     cname = cname.upper();
 
-    codec = QTextCodec::loadCharmapFile((QString)(dir + cname.data()));
+    const QString basicName = QString::fromLatin1(cname);
+    kdDebug() << k_funcinfo << endl << " Trying to find " << cname << " in " << dir << endl;
+    
+    QString charMapFileName;
+    bool gzipped = false; 
+    QDir qdir(dir);
+    if (!qdir.exists()) {
+        // The directory for the charmaps does not even exist... (That is common!)
+    }
+    else if (qdir.exists(basicName, false)) {
+        charMapFileName = basicName;
+    }
+    else if (qdir.exists(basicName+".gz", false)) {
+        charMapFileName = basicName + ".gz";
+        gzipped = true;
+    }
+    else {
+        // Check if we are asking a code page
+        // If yes, then check "CP99999" and "IBM99999"
+        // First we need to find the number of the codepage
+        QRegExp regexp("^(X-)?(CP|IBM)(-| )?(0-9)+");
+        if ( regexp.search(basicName) != -1) {
+            const QString num = regexp.cap(4);
+            if (num.isEmpty()) {
+                // No number, not a code page (or something went wrong)
+            }
+            else if (qdir.exists("IBM"+num)) {
+                charMapFileName = "IBM"+num;
+            }
+            else if (qdir.exists("IBM"+num+".gz")) {
+                charMapFileName = "IBM"+num+".gz";
+                gzipped = true;
+            }
+            else if (qdir.exists("CP"+num)) {
+                charMapFileName = "CP"+num;
+            }
+            else if (qdir.exists("CP"+num+".gz")) {
+                charMapFileName = "CP"+num+".gz";
+                gzipped = true;
+            }
+        }
+    }
+    
+    if (gzipped && !charMapFileName.isEmpty()) {
+        KQIODeviceGZip gzip(dir + "/" + charMapFileName);
+        if (gzip.open(IO_ReadOnly)) {
+            kdDebug() << "Loading gzipped charset..." << endl;
+            codec = QTextCodec::loadCharmap(&gzip);
+            gzip.close();
+        }
+        else
+            kdWarning() << "Could not open gzipped charset!" << endl;
+    }
+    else if (!charMapFileName.isEmpty()) {
+        codec = QTextCodec::loadCharmapFile(dir + "/" + charMapFileName);
+    }
 
     if(codec) {
         d->codecForNameDict.replace(key, codec);
@@ -622,8 +669,7 @@ QTextCodec *KCharsets::codecForName(const QString &n, bool &ok) const
 
     // this also failed, the last resort is now to take some compatibility charmap
 
-    cname = cname.lower();
-    cname = kcharsets_array_search< ConversionHints, const char* >( conversion_hints, (const char*)cname );
+    cname = kcharsets_array_search< ConversionHints, const char* >( conversion_hints, (const char*)name.data() );
 
     if(!cname.isEmpty())
         codec = QTextCodec::codecForName(cname);
