@@ -12,8 +12,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include <config.h>
@@ -26,10 +26,11 @@
 #include <string.h>
 #include <unistd.h>
 
-#if HAVE_LIMITS_H
+#ifdef HAVE_LIMITS_H
 #include <limits.h>
 #endif
 
+#include <qdir.h>
 #include <qfile.h>
 #include <qasciidict.h>
 #include <qstrlist.h>
@@ -48,6 +49,10 @@
 #define DISPLAY "QWS_DISPLAY"
 #endif
 
+#ifdef Q_WS_WIN
+#include <win32_utils.h>
+#endif
+
 template class QAsciiDict<QCString>;
 template class QPtrList<KCmdLineArgs>;
 
@@ -179,6 +184,9 @@ KCmdLineArgs::init(int _argc, char **_argv, const KAboutData *_about, bool noKAp
    parsed = false;
    mCwd = mCwdd.setObject(mCwd, new char [PATH_MAX+1], true);
    getcwd(mCwd, PATH_MAX);
+#ifdef Q_WS_WIN
+   win32_slashify(mCwd, PATH_MAX);
+#endif
    if (!noKApp)
       KApplication::addCmdLineOptions();
 }
@@ -196,7 +204,7 @@ const char * KCmdLineArgs::appName()
 
 void
 KCmdLineArgs::addCmdLineOptions( const KCmdLineOptions *options, const char *name,
-				 const char *id, const char *afterId)
+         const char *id, const char *afterId)
 {
    if (!argsList)
       argsList = new KCmdLineArgsList();
@@ -214,7 +222,7 @@ KCmdLineArgs::addCmdLineOptions( const KCmdLineOptions *options, const char *nam
          return; // Options already present.
 
       if (id && args->id && (::qstrcmp(id, args->id) == 0))
-	 return; // Options already present.
+   return; // Options already present.
 
       if (afterId && args->id && (::qstrcmp(afterId, args->id) == 0))
          pos = i+1;
@@ -259,9 +267,6 @@ KCmdLineArgs::loadAppArgs( QDataStream &ds)
    removeArgs("qt");
    removeArgs("kde");
 
-   if (ds.atEnd())
-      return;
-
    KCmdLineArgs *args;
    if ( argsList ) {
       for(args = argsList->first(); args; args = argsList->next())
@@ -270,6 +275,9 @@ KCmdLineArgs::loadAppArgs( QDataStream &ds)
       }
    }
 
+   if (ds.atEnd())
+      return;
+
    QCString qCwd;
    ds >> qCwd;
    delete [] mCwd;
@@ -294,6 +302,7 @@ KCmdLineArgs::loadAppArgs( QDataStream &ds)
        }
      }
    }
+   parsed = true;
 }
 
 KCmdLineArgs *KCmdLineArgs::parsedArgs(const char *id)
@@ -310,14 +319,6 @@ KCmdLineArgs *KCmdLineArgs::parsedArgs(const char *id)
       args = argsList->next();
    }
 
-   if (!args)
-   {
-#ifndef NDEBUG
-      fprintf(stderr, "WARNING (KCmdLineArgs):\n");
-      fprintf(stderr, "Application requests for parsedArgs(\"%s\") without a prior call\n", id?id:"null");
-      fprintf(stderr, "to addCmdLineOptions( ..., \"%s\"), or after a reset()\n\n", id?id:"null");
-#endif
-   }
    return args;
 }
 
@@ -342,9 +343,9 @@ void KCmdLineArgs::removeArgs(const char *id)
 /*
  * @return:
  *  0 - option not found.
- *  1 - option found			// -fork
- *  2 - inverse option found ('no')	// -nofork
- *  3 - option + arg found		// -fork now
+ *  1 - option found      // -fork
+ *  2 - inverse option found ('no') // -nofork
+ *  3 - option + arg found    // -fork now
  *
  *  +4 - no more options follow         // !fork
  */
@@ -591,7 +592,7 @@ KCmdLineArgs::parseAllArgs()
             printQ( QString("Qt: %1\n").arg(qVersion()));
             printQ( QString("KDE: %1\n").arg(KDE_VERSION_STRING));
             printQ( QString("%1: %2\n").
-			arg(about->programName()).arg(about->version()));
+      arg(about->programName()).arg(about->version()));
             exit(0);
          } else if ( (::qstrcmp(option, "license") == 0) )
          {
@@ -601,29 +602,40 @@ KCmdLineArgs::parseAllArgs()
             exit(0);
          } else if ( ::qstrcmp( option, "author") == 0 ) {
              enable_i18n();
-	     if ( about ) {
-		 const QValueList<KAboutPerson> authors = about->authors();
-		 if ( !authors.isEmpty() ) {
-                     QString authorlist;
-		     for (QValueList<KAboutPerson>::ConstIterator it = authors.begin(); it != authors.end(); ++it ) {
-			 QString email;
-			 if ( !(*it).emailAddress().isEmpty() )
-				 email = " <" + (*it).emailAddress() + ">";
-			 authorlist += QString("    ") + (*it).name() + email + "\n";
-		     }
-		     printQ( i18n("the 2nd argument is a list of name+address, one on each line","%1 was written by\n%2").arg ( QString(about->programName()) ).arg( authorlist ) );
-		 }
-	     } else {
-		 printQ( i18n("%1 was written by somebody who wants to remain anonymous.").arg(about->programName()) );
-	     }
-	     if (!about->bugAddress().isEmpty())
-	     {
-		if (about->bugAddress() == "submit@bugs.kde.org")
-		    printQ( i18n( "Please use http://bugs.kde.org to report bugs, do not mail the authors directly.\n" ) );
-		else
-		    printQ( i18n( "Please use %1 to report bugs, do not mail the authors directly.\n" ).arg(about->bugAddress()) );
-	     }
-	     exit(0);
+       if ( about ) {
+         const QValueList<KAboutPerson> authors = about->authors();
+         if ( !authors.isEmpty() ) {
+           QString authorlist;
+           for (QValueList<KAboutPerson>::ConstIterator it = authors.begin(); it != authors.end(); ++it ) {
+             QString email;
+             if ( !(*it).emailAddress().isEmpty() )
+               email = " <" + (*it).emailAddress() + ">";
+             authorlist += QString("    ") + (*it).name() + email + "\n";
+           }
+           printQ( i18n("the 2nd argument is a list of name+address, one on each line","%1 was written by\n%2").arg ( QString(about->programName()) ).arg( authorlist ) );
+         }
+       } else {
+         printQ( i18n("This application was written by somebody who wants to remain anonymous.") );
+       }
+       if (about)
+       {
+         if (!about->customAuthorTextEnabled ())
+         {
+           if (about->bugAddress().isEmpty() || about->bugAddress() == "submit@bugs.kde.org" )
+             printQ( i18n( "Please use http://bugs.kde.org to report bugs.\n" ) );
+           else {
+             if( about->authors().count() == 1 && about->authors().first().emailAddress() == about->bugAddress() )
+               printQ( i18n( "Please report bugs to %1.\n" ).arg( about->authors().first().emailAddress() ) );
+             else
+               printQ( i18n( "Please report bugs to %1.\n" ).arg(about->bugAddress()) );
+           }
+         }
+         else
+         {
+           printQ(about->customAuthorPlainText());
+         }
+       }
+       exit(0);
          } else {
            if ((option[0] == 'n') && (option[1] == 'o'))
            {
@@ -665,8 +677,11 @@ KCmdLineArgs::qt_argc()
    if (!argsList)
       KApplication::addCmdLineOptions(); // Lazy bastards!
 
-   KCmdLineArgs *args = parsedArgs("qt");
+   static int qt_argc = -1;
+   if( qt_argc != -1 )
+      return &qt_argc;
 
+   KCmdLineArgs *args = parsedArgs("qt");
    assert(args); // No qt options have been added!
    if (!argv)
    {
@@ -678,8 +693,8 @@ KCmdLineArgs::qt_argc()
    }
 
    assert(argc >= (args->count()+1));
-   argc = args->count() +1;
-   return &argc;
+   qt_argc = args->count() +1;
+   return &qt_argc;
 }
 
 /**
@@ -693,6 +708,10 @@ KCmdLineArgs::qt_argv()
    if (!argsList)
       KApplication::addCmdLineOptions(); // Lazy bastards!
 
+   static char** qt_argv;
+   if( qt_argv != NULL )
+      return &qt_argv;
+
    KCmdLineArgs *args = parsedArgs("qt");
    assert(args); // No qt options have been added!
    if (!argv)
@@ -704,14 +723,16 @@ KCmdLineArgs::qt_argv()
       exit(255);
    }
 
+   qt_argv = new char*[ args->count() + 2 ];
+   qt_argv[ 0 ] = qstrdup( appName());
    int i = 0;
    for(; i < args->count(); i++)
    {
-      argv[i+1] = (char *) args->arg(i);
+      qt_argv[i+1] = qstrdup((char *) args->arg(i));
    }
-   argv[i+1] = 0;
+   qt_argv[i+1] = 0;
 
-   return &argv;
+   return &qt_argv;
 }
 
 void
@@ -719,12 +740,12 @@ KCmdLineArgs::enable_i18n()
 {
     // called twice or too late
     if (KGlobal::_locale)
-	    return;
+      return;
 
     if (!KGlobal::_instance) {
-	KInstance *instance = new KInstance(about);
-	(void) instance->config();
-	// Don't delete instance!
+  KInstance *instance = new KInstance(about);
+  (void) instance->config();
+  // Don't delete instance!
     }
 }
 
@@ -734,7 +755,7 @@ KCmdLineArgs::usage(const QString &error)
     assert(KGlobal::_locale);
     QCString localError = error.local8Bit();
     if (localError[error.length()-1] == '\n')
-	localError = localError.left(error.length()-1);
+  localError = localError.left(error.length()-1);
     fprintf(stderr, "%s: %s\n", argv[0], localError.data());
 
     QString tmp = i18n("Use --help to get a list of available command line options.");
@@ -750,8 +771,8 @@ KCmdLineArgs::usage(const char *id)
    assert(argsList != 0); // It's an error to call usage(...) without
                           // having done addCmdLineOptions first!
 
-   QString optionFormatString		= "  %1 %2\n";
-   QString optionFormatStringDef	= "  %1 %2 [%3]\n";
+   QString optionFormatString   = "  %1 %2\n";
+   QString optionFormatStringDef  = "  %1 %2 [%3]\n";
    QString optionHeaderString = i18n("\n%1:\n");
    QString tmp;
    QString usage;
@@ -898,9 +919,9 @@ KCmdLineArgs::usage(const char *id)
 
             name = name.mid(1);
             if ((name[0] == '[') && (name[name.length()-1] == ']'))
-	       name = name.mid(1, name.length()-2);
+         name = name.mid(1, name.length()-2);
             printQ(optionFormatString.arg(name, -25)
-		 .arg(description));
+     .arg(description));
          }
          else
          {
@@ -1051,8 +1072,8 @@ KCmdLineArgs::setOption(const QCString &opt, bool enabled)
       addArgument(arg);
    }
    if (!parsedOptionList) {
- 	parsedOptionList = new KCmdLineParsedOptions;
-	parsedOptionList->setAutoDelete(true);
+  parsedOptionList = new KCmdLineParsedOptions;
+  parsedOptionList->setAutoDelete(true);
    }
 
    if (enabled)
@@ -1072,15 +1093,17 @@ KCmdLineArgs::setOption(const QCString &opt, const char *value)
       addArgument(arg);
       addArgument(value);
 
+#ifdef Q_WS_X11
       // Hack coming up!
       if (arg == "-display")
       {
          setenv(DISPLAY, value, true);
       }
+#endif
    }
    if (!parsedOptionList) {
-	parsedOptionList = new KCmdLineParsedOptions;
-	parsedOptionList->setAutoDelete(true);
+  parsedOptionList = new KCmdLineParsedOptions;
+  parsedOptionList->setAutoDelete(true);
    }
 
    parsedOptionList->insert( opt, new QCString(value) );
@@ -1221,20 +1244,21 @@ KCmdLineArgs::url(int n) const
    return makeURL( arg(n) );
 }
 
-KURL KCmdLineArgs::makeURL( const char *urlArg )
+KURL KCmdLineArgs::makeURL(const char *_urlArg)
 {
-   if (*urlArg == '/')
+   QString urlArg = QFile::decodeName(_urlArg);
+   if (!QDir::isRelativePath(urlArg))
    {
       KURL result;
-      result.setPath(QFile::decodeName( urlArg));
+      result.setPath(urlArg);
       return result; // Absolute path.
    }
 
-   if ( !KURL::isRelativeURL( QString::fromLocal8Bit(urlArg) ) )
-     return KURL(QString::fromLocal8Bit(urlArg)); // Argument is a URL
+   if ( !KURL::isRelativeURL(urlArg) )
+     return KURL(urlArg); // Argument is a URL
 
    KURL result;
-   result.setPath( cwd()+"/"+QFile::decodeName( urlArg ));
+   result.setPath( cwd()+"/"+urlArg );
    result.cleanPath();
    return result;  // Relative path
 }
@@ -1247,3 +1271,23 @@ KCmdLineArgs::addArgument(const char *argument)
 
    parsedArgList->append(argument);
 }
+
+static const KCmdLineOptions kde_tempfile_option[] =
+{
+   { "tempfile",       I18N_NOOP("The files/URLs opened by the application will be deleted after use"), 0},
+   KCmdLineLastOption
+};
+
+void
+KCmdLineArgs::addTempFileOption()
+{
+    KCmdLineArgs::addCmdLineOptions( kde_tempfile_option, "KDE-tempfile", "kde-tempfile" );
+}
+
+bool KCmdLineArgs::isTempFileSet()
+{
+    KCmdLineArgs* args = KCmdLineArgs::parsedArgs( "kde-tempfile" );
+    if ( args )
+        return args->isSet( "tempfile" );
+    return false;
+}
