@@ -15,8 +15,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #include <qcolor.h>
@@ -813,6 +813,40 @@ QVariant KConfigSkeleton::ItemStringList::property() const
 }
 
 
+KConfigSkeleton::ItemPathList::ItemPathList( const QString &group, const QString &key,
+                                            QStringList &reference,
+                                            const QStringList &defaultValue )
+  : ItemStringList( group, key, reference, defaultValue )
+{
+}
+
+void KConfigSkeleton::ItemPathList::readConfig( KConfig *config )
+{
+  config->setGroup( mGroup );
+  if ( !config->hasKey( mKey ) )
+    mReference = mDefault;
+  else
+    mReference = config->readPathListEntry( mKey );
+  mLoadedValue = mReference;
+
+  readImmutability( config );
+}
+
+void KConfigSkeleton::ItemPathList::writeConfig( KConfig *config )
+{
+  if ( mReference != mLoadedValue ) // WABA: Is this test needed?
+  {
+    config->setGroup( mGroup );
+    if ((mDefault == mReference) && !config->hasDefault( mKey))
+      config->revertToDefault( mKey );
+    else {
+      QStringList sl = mReference;
+      config->writePathEntry( mKey, sl );
+    }
+  }
+}
+
+
 KConfigSkeleton::ItemIntList::ItemIntList( const QString &group, const QString &key,
                                       QValueList<int> &reference,
                                       const QValueList<int> &defaultValue )
@@ -918,6 +952,7 @@ void KConfigSkeleton::readConfig()
   
   QString origGroup = mConfig->group();
 
+  mConfig->reparseConfiguration();
   KConfigSkeletonItem::List::ConstIterator it;
   for( it = mItems.begin(); it != mItems.end(); ++it )
   {
