@@ -1,6 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -13,22 +13,23 @@
  *
  *  You should have received a copy of the GNU Library General Public License
  *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- *  Boston, MA 02111-1307, USA.
+ *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ *  Boston, MA 02110-1301, USA.
  **/
 
 #ifndef KMSPECIALPRINTERDLG_H
 #define KMSPECIALPRINTERDLG_H
 
-#include <kdialog.h>
+#include <kdialogbase.h>
 
 class KMPrinter;
 class QLineEdit;
 class QCheckBox;
+class QComboBox;
 class KIconButton;
-class KEditListBox;
+class KXmlCommandSelector;
 
-class KMSpecialPrinterDlg : public KDialog
+class KMSpecialPrinterDlg : public KDialogBase
 {
 	Q_OBJECT
 public:
@@ -41,14 +42,16 @@ protected:
 	bool checkSettings();
 
 protected slots:
-	virtual void done(int);
-	void slotBrowse();
+	void slotOk();
+	void slotTextChanged(const QString &);
 
 private:
-	QLineEdit	*m_name, *m_description, *m_location, *m_command, *m_extension;
+	QLineEdit	*m_name, *m_description, *m_location, *m_extension;
+	QComboBox	*m_mimetype;
 	QCheckBox	*m_usefile;
+	QStringList	m_mimelist;
 	KIconButton	*m_icon;
-	KEditListBox	*m_require;
+	KXmlCommandSelector	*m_command;
 };
 
 #endif
