@@ -16,15 +16,15 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 
 */
 
 #include <qlayout.h>
 
 #include <kaboutdata.h>
-#include <kconfigskeleton.h> 
+#include <kconfigskeleton.h>
 #include <kconfigdialogmanager.h>
 #include <kdebug.h>
 #include <kglobal.h>
@@ -40,7 +40,8 @@ public:
     KCModulePrivate():
         _about( 0 ),
         _useRootOnlyMsg( false ),
-        _hasOwnInstance( true )
+        _hasOwnInstance( true ),
+        _unmanagedWidgetChangeState( false )
         { }
 
     KInstance *_instance;
@@ -51,6 +52,11 @@ public:
     QPtrList<KConfigDialogManager> managers;
     QString _quickHelp;
 
+    // this member is used to record the state on non-automatically
+    // managed widgets, allowing for mixed KConfigXT-drive and manual
+    // widgets to coexist peacefully and do the correct thing with
+    // the changed(bool) signal
+    bool _unmanagedWidgetChangeState;
 };
 
 KCModule::KCModule(QWidget *parent, const char *name, const QStringList &)
@@ -79,7 +85,7 @@ KCModule::KCModule(KInstance *instance, QWidget *parent, const QStringList & )
 }
 
 void KCModule::init()
-{ 
+{
     d = new KCModulePrivate;
    _btn = Help|Default|Apply;
 }
@@ -124,11 +130,25 @@ void KCModule::defaults()
 
 void KCModule::widgetChanged()
 {
-    bool bChanged = false;
+    emit changed(d->_unmanagedWidgetChangeState || managedWidgetChangeState());
+}
+
+bool KCModule::managedWidgetChangeState() const
+{
     KConfigDialogManager* manager;
     for( manager = d->managers.first(); manager; manager = d->managers.next() )
-        bChanged |= manager->hasChanged();
-    emit( changed( bChanged ));
+    {
+        if ( manager->hasChanged() )
+            return true;
+    }
+
+    return false;
+}
+
+void KCModule::unmanagedWidgetChangeState(bool changed)
+{
+    d->_unmanagedWidgetChangeState = changed;
+    widgetChanged();
 }
 
 const KAboutData *KCModule::aboutData() const
