@@ -16,8 +16,8 @@
 
    You should have received a copy of the GNU Lesser General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include <qclipboard.h>
@@ -46,9 +46,8 @@
 class KComboBox::KComboBoxPrivate
 {
 public:
-    KComboBoxPrivate()
+    KComboBoxPrivate() : klineEdit(0L)
     {
-        klineEdit = 0L;
     }
     ~KComboBoxPrivate()
     {
@@ -58,13 +57,13 @@ public:
 };
 
 KComboBox::KComboBox( QWidget *parent, const char *name )
-    : QComboBox( parent, name )
+    : QComboBox( parent, name ), d(new KComboBoxPrivate)
 {
     init();
 }
 
 KComboBox::KComboBox( bool rw, QWidget *parent, const char *name )
-    : QComboBox( rw, parent, name )
+    : QComboBox( rw, parent, name ), d(new KComboBoxPrivate)
 {
     init();
 
@@ -78,13 +77,10 @@ KComboBox::KComboBox( bool rw, QWidget *parent, const char *name )
 KComboBox::~KComboBox()
 {
     delete d;
-    d = 0;
 }
 
 void KComboBox::init()
 {
-    d = new KComboBoxPrivate;
-
     // Permanently set some parameters in the parent object.
     QComboBox::setAutoCompletion( false );
 
@@ -99,7 +95,8 @@ bool KComboBox::contains( const QString& _text ) const
     if ( _text.isEmpty() )
         return false;
 
-    for (int i = 0; i < count(); i++ )
+    const int itemCount = count();
+    for (int i = 0; i < itemCount; ++i )
     {
         if ( text(i) == _text )
             return true;
@@ -165,7 +162,7 @@ void KComboBox::makeCompletion( const QString& text )
         if( text.isNull() || !listBox() )
             return;
 
-        int index = listBox()->index( listBox()->findItem( text ) );
+        const int index = listBox()->index( listBox()->findItem( text ) );
         if( index >= 0 )
             setCurrentItem( index );
     }
@@ -251,7 +248,7 @@ void KComboBox::wheelEvent( QWheelEvent *ev )
 void KComboBox::setLineEdit( QLineEdit *edit )
 {
     if ( !editable() && edit &&
-         qstrcmp( edit->className(), "QLineEdit" ) == 0 )
+         !qstrcmp( edit->className(), "QLineEdit" ) )
     {
         // uic generates code that creates a read-only KComboBox and then
         // calls combo->setEditable( true ), which causes QComboBox to set up
@@ -309,7 +306,8 @@ void KComboBox::setCurrentItem( const QString& item, bool insert, int index )
 {
     int sel = -1;
 
-    for (int i = 0; i < count(); ++i)
+    const int itemCount = count();
+    for (int i = 0; i < itemCount; ++i)
     {
         if (text(i) == item)
         {
@@ -348,7 +346,7 @@ void KComboBox::lineEditDeleted()
 
 // we are always read-write
 KHistoryCombo::KHistoryCombo( QWidget *parent, const char *name )
-    : KComboBox( true, parent, name )
+    : KComboBox( true, parent, name ), d(0)
 {
     init( true ); // using completion
 }
@@ -356,7 +354,7 @@ KHistoryCombo::KHistoryCombo( QWidget *parent, const char *name )
 // we are always read-write
 KHistoryCombo::KHistoryCombo( bool useCompletion,
                               QWidget *parent, const char *name )
-    : KComboBox( true, parent, name )
+    : KComboBox( true, parent, name ), d(0)
 {
     init( useCompletion );
 }
@@ -393,8 +391,15 @@ void KHistoryCombo::setHistoryItems( QStringList items,
     KComboBox::clear();
 
     // limit to maxCount()
-    while ( (int) items.count() > maxCount() && !items.isEmpty() )
-        items.remove( items.begin() );
+    const int itemCount = items.count(); 
+    const int toRemove = itemCount - maxCount();
+
+    if (toRemove >= itemCount) {
+        items.clear();
+    } else {
+        for (int i = 0; i < toRemove; ++i) 
+            items.pop_front();
+    }
 
     insertItems( items );
 
@@ -412,7 +417,8 @@ void KHistoryCombo::setHistoryItems( QStringList items,
 QStringList KHistoryCombo::historyItems() const
 {
     QStringList list;
-    for ( int i = 0; i < count(); i++ )
+    const int itemCount = count();
+    for ( int i = 0; i < itemCount; ++i )
         list.append( text( i ) );
 
     return list;
@@ -420,7 +426,7 @@ QStringList KHistoryCombo::historyItems() const
 
 void KHistoryCombo::clearHistory()
 {
-    QString temp = currentText();
+    const QString temp = currentText();
     KComboBox::clear();
     if ( useCompletion() )
         completionObject()->clear();
@@ -432,7 +438,7 @@ void KHistoryCombo::addContextMenuItems( QPopupMenu* menu )
     if ( menu )
     {
         menu->insertSeparator();
-        int id = menu->insertItem( SmallIcon("history_clear"), i18n("Clear &History"), this, SLOT( slotClear()));
+        int id = menu->insertItem( SmallIconSet("history_clear"), i18n("Clear &History"), this, SLOT( slotClear()));
         if (!count())
            menu->setItemEnabled(id, false);
     }
@@ -447,11 +453,16 @@ void KHistoryCombo::addToHistory( const QString& item )
     bool wasCurrent = false;
     // remove all existing items before adding
     if ( !duplicatesEnabled() ) {
-        for ( int i = 0; i < count(); i++ ) {
+        int i = 0;
+        int itemCount = count();
+        while ( i < itemCount ) {
             if ( text( i ) == item ) {
                 if ( !wasCurrent )
                   wasCurrent = ( i == currentItem() );
                 removeItem( i );
+                --itemCount;
+            } else {
+                ++i;
             }
         }
     }
@@ -465,17 +476,18 @@ void KHistoryCombo::addToHistory( const QString& item )
     if ( wasCurrent )
         setCurrentItem( 0 );
 
-    int last;
-    QString rmItem;
+    const bool useComp = useCompletion();
 
-    bool useComp = useCompletion();
-    while ( count() > maxCount() && count() > 0 ) {
+    const int last = count() - 1; // last valid index
+    const int mc = maxCount();
+    const int stopAt = QMAX(mc, 0);
+
+    for (int rmIndex = last; rmIndex >= stopAt; --rmIndex) {
         // remove the last item, as long as we are longer than maxCount()
         // remove the removed item from the completionObject if it isn't
         // anymore available at all in the combobox.
-        last = count() - 1;
-        rmItem = text( last );
-        removeItem( last );
+        const QString rmItem = text( rmIndex );
+        removeItem( rmIndex );
         if ( useComp && !contains( rmItem ) )
             completionObject()->removeItem( rmItem );
     }
@@ -490,11 +502,16 @@ bool KHistoryCombo::removeFromHistory( const QString& item )
         return false;
 
     bool removed = false;
-    QString temp = currentText();
-    for ( int i = 0; i < count(); i++ ) {
-        while ( item == text( i ) ) {
+    const QString temp = currentText();
+    int i = 0;
+    int itemCount = count();
+    while ( i < itemCount ) {
+        if ( item == text( i ) ) {
             removed = true;
             removeItem( i );
+            --itemCount;
+        } else {
+            ++i;
         }
     }
 
@@ -511,13 +528,16 @@ void KHistoryCombo::rotateUp()
     if ( myIterateIndex == -1 )
         myText = currentText();
 
-    myIterateIndex++;
+    ++myIterateIndex;
 
     // skip duplicates/empty items
-    while ( myIterateIndex < count()-1 &&
-            (currentText() == text( myIterateIndex ) ||
+    const int last = count() - 1; // last valid index
+    const QString currText = currentText();
+
+    while ( myIterateIndex < last &&
+            (currText == text( myIterateIndex ) ||
              text( myIterateIndex ).isEmpty()) )
-        myIterateIndex++;
+        ++myIterateIndex;
 
     if ( myIterateIndex >= count() ) {
         myRotated = true;
@@ -539,13 +559,14 @@ void KHistoryCombo::rotateDown()
     if ( myIterateIndex == -1 )
         myText = currentText();
 
-    myIterateIndex--;
+    --myIterateIndex;
 
+    const QString currText = currentText();
     // skip duplicates/empty items
     while ( myIterateIndex >= 0 &&
-            (currentText() == text( myIterateIndex ) ||
+            (currText == text( myIterateIndex ) ||
              text( myIterateIndex ).isEmpty()) )
-        myIterateIndex--;
+        --myIterateIndex;
 
 
     if ( myIterateIndex < 0 ) {
@@ -556,7 +577,7 @@ void KHistoryCombo::rotateDown()
         }
         else { // bottom of history
             if ( myIterateIndex == -2 ) {
-                KNotifyClient::event( winId(), KNotifyClient::notification,
+                KNotifyClient::event( (int)winId(), KNotifyClient::notification,
                                       i18n("No further item in the history."));
             }
 
@@ -575,12 +596,12 @@ void KHistoryCombo::keyPressEvent( QKeyEvent *e )
     KKey event_key( e );
 
     // going up in the history, rotating when reaching QListBox::count()
-    if ( KKey(KStdAccel::rotateUp().keyCodeQt()) == event_key )
+    if ( KStdAccel::rotateUp().contains(event_key) )
         rotateUp();
 
     // going down in the history, no rotation possible. Last item will be
     // the text that was in the lineedit before Up was called.
-    else if ( KKey(KStdAccel::rotateDown().keyCodeQt()) == event_key )
+    else if ( KStdAccel::rotateDown().contains(event_key) )
         rotateDown();
     else
         KComboBox::keyPressEvent( e );
@@ -589,7 +610,7 @@ void KHistoryCombo::keyPressEvent( QKeyEvent *e )
 void KHistoryCombo::wheelEvent( QWheelEvent *ev )
 {
     // Pass to poppable listbox if it's up
-    QListBox *lb = listBox();
+    QListBox* const lb = listBox();
     if ( lb && lb->isVisible() )
     {
         QApplication::sendEvent( lb, ev );
@@ -631,10 +652,11 @@ void KHistoryCombo::setPixmapProvider( KPixmapProvider *prov )
 
 void KHistoryCombo::insertItems( const QStringList& items )
 {
-    QStringList::ConstIterator it = items.begin();
-    QString item;
-    while ( it != items.end() ) {
-        item = *it;
+    QStringList::ConstIterator it = items.constBegin();
+    const QStringList::ConstIterator itEnd = items.constEnd();
+
+    while ( it != itEnd ) {
+        const QString item = *it;
         if ( !item.isEmpty() ) { // only insert non-empty items
             if ( myPixProvider )
                 insertItem( myPixProvider->pixmapFor(item, KIcon::SizeSmall),
