@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU Library General Public
  *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include <khtmlview.h>
@@ -216,7 +216,7 @@ Value DOMNode::getValueProperty(ExecState *exec, int token) const
   case OnKeyDown:
     return getListener(DOM::EventImpl::KEYDOWN_EVENT);
   case OnKeyPress:
-    return getListener(DOM::EventImpl::KHTML_KEYPRESS_EVENT);
+    return getListener(DOM::EventImpl::KEYPRESS_EVENT);
   case OnKeyUp:
     return getListener(DOM::EventImpl::KEYUP_EVENT);
   case OnLoad:
@@ -360,7 +360,7 @@ void DOMNode::putValueProperty(ExecState *exec, int token, const Value& value, i
     setListener(exec,DOM::EventImpl::KEYDOWN_EVENT,value);
     break;
   case OnKeyPress:
-    setListener(exec,DOM::EventImpl::KHTML_KEYPRESS_EVENT,value);
+    setListener(exec,DOM::EventImpl::KEYPRESS_EVENT,value);
     break;
   case OnKeyUp:
     setListener(exec,DOM::EventImpl::KEYUP_EVENT,value);
@@ -401,36 +401,40 @@ void DOMNode::putValueProperty(ExecState *exec, int token, const Value& value, i
   case OnUnload:
     setListener(exec,DOM::EventImpl::UNLOAD_EVENT,value);
     break;
-  case ScrollTop: {
-    khtml::RenderObject *rend = node.handle() ? node.handle()->renderer() : 0L;
-    if (rend && rend->layer()) {
-        node.handle()->getDocument()->updateLayout();
-        if (rend->style()->hidesOverflow())
-            rend->layer()->scrollToYOffset(value.toInt32(exec));
-        else if (rend->isRoot()) {
-            QScrollView* sview = node.ownerDocument().view();
-            if (sview)
-                sview->setContentsPos(sview->contentsX(), value.toInt32(exec));
-        }
-    }       
-    break;
-  }
-  case ScrollLeft: {
-    khtml::RenderObject *rend = node.handle() ? node.handle()->renderer() : 0L;
-    if (rend && rend->layer()) {
-        node.handle()->getDocument()->updateLayout();
-        if (rend->style()->hidesOverflow())
-            rend->layer()->scrollToXOffset(value.toInt32(exec));
-        else if (rend->isRoot()) {
-            QScrollView* sview = node.ownerDocument().view();
-            if (sview)
-                sview->setContentsPos(value.toInt32(exec), sview->contentsY());
-        }
-    }
-    break;
-  }
   default:
-    kdDebug(6070) << "WARNING: DOMNode::putValueProperty unhandled token " << token << endl;
+    // Make sure our layout is up to date 
+    DOM::DocumentImpl* docimpl = node.handle()->getDocument();
+    if (docimpl)
+      docimpl->updateLayout();
+
+    khtml::RenderObject *rend = node.handle() ? node.handle()->renderer() : 0L;
+
+    switch (token) {
+      case ScrollLeft:
+        if (rend && rend->layer()) {
+          if (rend->style()->hidesOverflow())
+            rend->layer()->scrollToXOffset(value.toInt32(exec));
+          else if (rend->isRoot()) {
+            QScrollView* sview = node.ownerDocument().view();
+            if (sview)
+              sview->setContentsPos(value.toInt32(exec), sview->contentsY());
+          }
+        }
+        break;
+      case ScrollTop:
+        if (rend && rend->layer()) {
+          if (rend->style()->hidesOverflow())
+            rend->layer()->scrollToYOffset(value.toInt32(exec));
+          else if (rend->isRoot()) {
+            QScrollView* sview = node.ownerDocument().view();
+            if (sview)
+              sview->setContentsPos(sview->contentsX(), value.toInt32(exec));
+          }
+        }
+        break;
+      default:
+      kdDebug(6070) << "WARNING: DOMNode::putValueProperty unhandled token " << token << endl;
+    }
   }
 }
 
@@ -599,7 +603,7 @@ Value DOMNodeList::tryGet(ExecState *exec, const Identifier &p) const
 
   // Look in the prototype (for functions) before assuming it's an item's name
   Object proto = Object::dynamicCast(prototype());
-  if (!proto.isNull() && proto.hasProperty(exec,p))
+  if (proto.isValid() && proto.hasProperty(exec,p))
     return proto.get(exec,p);
 
   Value result;
@@ -839,7 +843,12 @@ Value DOMDocument::getValueProperty(ExecState *exec, int token) const
     //kdDebug() << "DOMDocument::StyleSheets, returning " << doc.styleSheets().length() << " stylesheets" << endl;
     return getDOMStyleSheetList(exec, doc.styleSheets(), doc);
   case DOMDocument::DefaultView: // DOM2
+    {
+    KHTMLView *view = node.handle()->getDocument()->view();
+    if (view)
+        return Window::retrieve(view->part());
     return getDOMAbstractView(exec, doc.defaultView());
+    }
   case PreferredStylesheetSet:
     return String(doc.preferredStylesheetSet());
   case SelectedStylesheetSet:
@@ -945,7 +954,7 @@ Value DOMDocumentProtoFunc::tryCall(ExecState *exec, Object &thisObj, const List
     }
     else {
       Object obj = Object::dynamicCast(args[2]);
-      if (!obj.isNull())
+      if (obj.isValid())
       {
         DOM::CustomNodeFilter *customFilter = new JSNodeFilter(obj);
         DOM::NodeFilter filter = DOM::NodeFilter::createCustom(customFilter);
@@ -1755,10 +1764,10 @@ Value DOMCharacterDataProtoFunc::tryCall(ExecState *exec, Object &thisObj, const
     case DOMCharacterData::ReplaceData:
       data.replaceData(args[0].toInteger(exec),args[1].toInteger(exec),args[2].toString(exec).string());
       return Undefined();
-      break;
     default:
-      return Undefined();
+      break;
   }
+  return Undefined();
 }
 
 // -------------------------------------------------------------------------
@@ -1792,8 +1801,8 @@ Value DOMTextProtoFunc::tryCall(ExecState *exec, Object &thisObj, const List &ar
   switch(id) {
     case DOMText::SplitText:
       return getDOMNode(exec,text.splitText(args[0].toInteger(exec)));
-      break;
     default:
-      return Undefined();
+      break;
   }
+  return Undefined();
 }
