@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU Library General Public
  *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef _KJS_EVENTS_H_
@@ -39,8 +39,6 @@ namespace KJS {
      */
     JSEventListener(Object _listener, ObjectImp *_compareListenerImp, const Object &_win, bool _html = false);
     virtual ~JSEventListener();
-    void hackSetThisObj( Object& thisObj ) { m_hackThisObj = thisObj; }
-    void hackUnsetThisObj() { m_hackThisObj = Object(0L); }
     virtual void handleEvent(DOM::Event &evt);
     virtual DOM::DOMString eventListenerType();
     // Return the KJS function object executed when this event is emitted
@@ -50,6 +48,7 @@ namespace KJS {
     // if it was added to a DOM node in another frame (#61467). But calling removeEventListener on
     // all nodes we're listening to is quite difficult.
     void clear() { listener = Object(); }
+    bool isHTMLEventListener() const { return html; }
 
   protected:
     mutable Object listener;
@@ -65,12 +64,12 @@ namespace KJS {
     // the correct event listener, as well as the 'listener.handleEvent' function, we need to call.
     mutable ObjectImp *compareListenerImp;
     bool html;
-    Object win, m_hackThisObj;
+    Object win;
   };
 
   class JSLazyEventListener : public JSEventListener {
   public:
-    JSLazyEventListener(const QString &_code, const QString &_name, const Object &_win, bool _html = false);
+    JSLazyEventListener(const QString &_code, const QString &_name, const Object &_win, DOM::NodeImpl* node);
     ~JSLazyEventListener();
     virtual void handleEvent(DOM::Event &evt);
     Object listenerObj() const;
@@ -80,6 +79,7 @@ namespace KJS {
     mutable QString code;
     mutable QString name;
     mutable bool parsed;
+    DOM::NodeImpl *originalNode;
   };
 
   // Constructor for Event - currently only used for some global vars
@@ -181,7 +181,8 @@ namespace KJS {
     // no put - all read-only
     virtual const ClassInfo* classInfo() const { return &info; }
     static const ClassInfo info;
-    enum { Key, VirtKey, OutputString, InitTextEvent, InputGenerated, NumPad };
+    enum { Key, VirtKey, OutputString, InitTextEvent, InputGenerated, NumPad,
+           CtrlKey, ShiftKey, AltKey, MetaKey };
     DOM::TextEvent toTextEvent() const { return static_cast<DOM::TextEvent>(event); }
   };
 
