@@ -21,8 +21,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 //----------------------------------------------------------------------------
 //
@@ -172,7 +172,7 @@ void HTMLTokenizer::reset()
         KHTML_DELETE_QCHAR_VEC(scriptCode);
     scriptCode = 0;
     scriptCodeSize = scriptCodeMaxSize = scriptCodeResync = 0;
-    
+
     if (m_autoCloseTimer) {
         killTimer(m_autoCloseTimer);
         m_autoCloseTimer = 0;
@@ -319,6 +319,8 @@ void HTMLTokenizer::parseSpecial(TokenizerString &src)
         unsigned char ch = src->latin1();
         if ( !scriptCodeResync && !brokenComments && !textarea && !xmp && !title && ch == '-' && scriptCodeSize >= 3 && !src.escaped() && QConstString( scriptCode+scriptCodeSize-3, 3 ).string() == "<!-" ) {
             comment = true;
+            scriptCode[ scriptCodeSize++ ] = ch;
+            ++src;
             parseComment( src );
             continue;
         }
@@ -443,33 +445,62 @@ void HTMLTokenizer::scriptExecution( const QString& str, const QString& scriptUR
 
 void HTMLTokenizer::parseComment(TokenizerString &src)
 {
+    // SGML strict
+    bool strict = parser->doc()->inStrictMode() && parser->doc()->htmlMode() != DocumentImpl::XHtml;
+    int delimiterCount = 0;
+    bool canClose = false;
+
     checkScriptBuffer(src.length());
     while ( src.length() ) {
         scriptCode[ scriptCodeSize++ ] = *src;
+
 #if defined(TOKEN_DEBUG) && TOKEN_DEBUG > 1
         qDebug("comment is now: *%s*",
                QConstString((QChar*)src.current(), kMin(16U, src.length())).string().latin1());
 #endif
-        if (src->unicode() == '>' &&
-            ( ( brokenComments && !( script || style ) ) ||
-              ( scriptCodeSize > 2 && scriptCode[scriptCodeSize-3] == '-' &&
-                scriptCode[scriptCodeSize-2] == '-' ) ) ) {
-            ++src;
-            if ( !( script || xmp || textarea || style) ) {
+
+        if (strict)
+        {
+            if (src->unicode() == '-') {
+                delimiterCount++;
+                if (delimiterCount == 2) {
+                    delimiterCount = 0;
+                    canClose = !canClose;
+                }
+            }
+            else
+                delimiterCount = 0;
+        }
+
+        if ((!strict || canClose) && src->unicode() == '>')
+        {
+            bool handleBrokenComments =  brokenComments && !( script || style );
+            bool scriptEnd=false;
+            if (!strict)
+            {
+                if ( scriptCodeSize > 2 && scriptCode[scriptCodeSize-3] == '-' &&
+                     scriptCode[scriptCodeSize-2] == '-' )
+                    scriptEnd=true;
+            }
+
+            if (canClose || handleBrokenComments || scriptEnd ){
+                ++src;
+                if ( !( script || xmp || textarea || style) ) {
 #ifdef COMMENTS_IN_DOM
-                checkScriptBuffer();
-                scriptCode[ scriptCodeSize ] = 0;
-                scriptCode[ scriptCodeSize + 1 ] = 0;
-                currToken.tid = ID_COMMENT;
-                processListing(DOMStringIt(scriptCode, scriptCodeSize - 2));
-                processToken();
-                currToken.tid = ID_COMMENT + ID_CLOSE_TAG;
-                processToken();
+                    checkScriptBuffer();
+                    scriptCode[ scriptCodeSize ] = 0;
+                    scriptCode[ scriptCodeSize + 1 ] = 0;
+                    currToken.tid = ID_COMMENT;
+                    processListing(DOMStringIt(scriptCode, scriptCodeSize - 2));
+                    processToken();
+                    currToken.tid = ID_COMMENT + ID_CLOSE_TAG;
+                    processToken();
 #endif
-                scriptCodeSize = 0;
+                    scriptCodeSize = 0;
+                }
+                comment = false;
+                return; // Finished parsing comment
             }
-            comment = false;
-            return; // Finished parsing comment
         }
         ++src;
     }
@@ -1056,6 +1087,14 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
 #if defined(TOKEN_DEBUG) && TOKEN_DEBUG > 0
             kdDebug( 6036 ) << "appending Tag: " << tagID << endl;
 #endif
+            // If the tag requires an end tag it cannot be flat,
+            // unless we are using the HTML parser to parse XHTML
+            // The only exception is SCRIPT and priority 0 tokens.
+            if (tagID < ID_CLOSE_TAG && tagID != ID_SCRIPT &&
+                DOM::endTag[tagID] == DOM::REQUIRED &&
+                parser->doc()->htmlMode() != DocumentImpl::XHtml)
+                currToken.flat = false;
+
             bool beginTag = !currToken.flat && (tagID < ID_CLOSE_TAG);
 
             if(tagID >= ID_CLOSE_TAG)
@@ -1102,7 +1141,8 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                         type.compare("text/jscript") != 0 &&
                         type.compare("text/ecmascript") != 0 &&
                         type.compare("text/livescript") != 0 &&
-			type.compare("application/x-javascript") != 0 )
+			type.compare("application/x-javascript") != 0 &&
+			type.compare("application/ecmascript") != 0 )
                         javascript = false;
                 } else if( a ) {
                     /*
@@ -1130,12 +1170,17 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
 
             processToken();
 
-            // lets see if we're still in parsing mood for spaces
-            pre = parser->preMode();
+            if ( parser->selectMode() && beginTag)
+                discard = AllDiscard;
 
             switch( tagID ) {
             case ID_PRE:
-                discard = LFDiscard;
+                pre = beginTag;
+                if (beginTag)
+                    discard = LFDiscard;
+                prePos = 0;
+                break;
+            case ID_BR:
                 prePos = 0;
                 break;
             case ID_SCRIPT:
@@ -1313,9 +1358,11 @@ void HTMLTokenizer::write( const TokenizerString &str, bool appendData )
         else if ( startTag )
         {
             startTag = false;
+            bool endTag = false;
 
             switch(cc) {
             case '/':
+                endTag = true;
                 break;
             case '!':
             {
@@ -1362,21 +1409,19 @@ void HTMLTokenizer::write( const TokenizerString &str, bool appendData )
             }
             }; // end case
 
-            if ( pending ) {
-                // pre context always gets its spaces/linefeeds
-                if ( pre )
+            // According to SGML any LF immediately after a starttag, or
+            // immediately before an endtag should be ignored.
+            // ### Gecko and MSIE though only ignores LF immediately after
+            // starttags and only for PRE elements -- asj (28/06-2005)
+            if ( pending )
+                if (!select)
                     addPending();
-                // only add in existing inline context or if
-                // we just started one, i.e. we're about to insert real text
-                else if ( !parser->selectMode() &&
-                          ( !parser->noSpaces() || dest > buffer )) {
-                    addPending();
-		    discard = AllDiscard;
-                }
-                // just forget it
                 else
                     pending = NonePending;
-            }
+
+            // Cancel unused discards
+            discard = NoneDiscard;
+            // if (!endTag) discard = LFDiscard;
 
             processToken();
 
@@ -1389,42 +1434,34 @@ void HTMLTokenizer::write( const TokenizerString &str, bool appendData )
             ++src;
             if ( pending )
                 addPending();
+            discard = NoneDiscard;
             parseEntity(src, dest, true);
         }
         else if ( cc == '<' && !src.escaped())
         {
             tagStartLineno = lineno+src.lineCount();
             ++src;
+            discard = NoneDiscard;
             startTag = true;
         }
         else if (( cc == '\n' ) || ( cc == '\r' ))
         {
-            if (select && !script)
-            {
-                if (discard == LFDiscard)
-                {
-                    // Ignore this LF
-                    discard = NoneDiscard; // We have discarded 1 LF
-                }
-                else if(discard == AllDiscard)
-                {
-                }
-                else
-                {
-                     // Process this LF
-                    if (pending == NonePending)
-                        pending = LFPending;
-                }
+            if (discard == SpaceDiscard)
+                discard = NoneDiscard;
+
+            if (discard == LFDiscard) {
+                // Ignore one LF
+                discard = NoneDiscard;
             }
-            else {
-                if (discard == LFDiscard || discard == AllDiscard)
-                {
-                    // Ignore this LF
-                    discard = NoneDiscard; // We have discarded 1 LF
-                }
-                else
-                {
-                    // Process this LF
+            else if (discard == AllDiscard)
+            {
+                // Ignore
+            }
+            else
+            {
+                if (select && !script) {
+                    pending = LFPending;
+                } else {
                     if (pending)
                         addPending();
                     pending = LFPending;
@@ -1440,25 +1477,29 @@ void HTMLTokenizer::write( const TokenizerString &str, bool appendData )
         }
         else if (( cc == ' ' ) || ( cc == '\t' ))
         {
-            if (select && !script) {
-                if(discard == SpaceDiscard)
-                    discard = NoneDiscard;
-                else if(discard == AllDiscard)
-                { }
-                else
-                        pending = SpacePending;
+            if(discard == LFDiscard)
+                discard = NoneDiscard;
 
+            if(discard == SpaceDiscard) {
+                // Ignore one space
+                discard = NoneDiscard;
+            }
+            else if(discard == AllDiscard)
+            {
+                // Ignore
             }
             else {
-                if (discard == AllDiscard)
-                    discard = NoneDiscard;
-
-                if (pending)
-                    addPending();
-                if (cc == ' ')
-                    pending = SpacePending;
-                else
-                    pending = TabPending;
+                if (select && !script) {
+                    if (!pending)
+                        pending = SpacePending;
+                } else {
+                    if (pending)
+                        addPending();
+                    if (cc == ' ')
+                        pending = SpacePending;
+                    else
+                        pending = TabPending;
+                }
             }
 
             ++src;
@@ -1495,8 +1536,8 @@ void HTMLTokenizer::setAutoClose( bool b ) {
     killTimer( m_autoCloseTimer );
     m_autoCloseTimer = 0;
     if ( b )
-        m_autoCloseTimer = startTimer(100);       
-}    
+        m_autoCloseTimer = startTimer(100);
+}
 
 void HTMLTokenizer::end()
 {
@@ -1633,8 +1674,6 @@ void HTMLTokenizer::processToken()
 
     if ( currToken.flat && currToken.tid != ID_TEXT && !parser->noSpaces() )
 	discard = NoneDiscard;
-    else if ( parser->selectMode() )
-        discard = AllDiscard;
 
     currToken.reset();
     if (jsProxy)
