@@ -17,8 +17,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 
     This class provides all functionality needed for loading images, style sheets and html
     pages from the web. It has a memory cache for these objects.
@@ -110,6 +110,7 @@ namespace khtml
             m_deleted = false;
             m_free = false;
             m_hadError = false;
+            m_wasBlocked = false;
             m_prev = m_next = 0;
 	}
         virtual ~CachedObject();
@@ -171,12 +172,14 @@ namespace khtml
         bool m_loading : 1;
         bool m_free : 1;
 	bool m_hadError : 1;
+	bool m_wasBlocked : 1;
 
     private:
         bool allowInLRUList() const { return canDelete() && !m_free && status() != Persistent; }
         CachedObject* m_next;
         CachedObject* m_prev;
         friend class Cache;
+        friend class ::KHTMLPart;
     };
 
 
@@ -265,8 +268,15 @@ namespace khtml
 
         bool isTransparent() const { return isFullyTransparent; }
         bool isErrorImage() const { return m_hadError; }
-        QString suggestedFilename() const { return m_suggestedFilename; }
+        bool isBlockedImage() const { return m_wasBlocked; }
+        const QString& suggestedFilename() const { return m_suggestedFilename; }
         void setSuggestedFilename( const QString& s ) { m_suggestedFilename = s;  }
+#ifdef IMAGE_TITLES
+        const QString& suggestedTitle() const { return m_suggestedTitle; }
+        void setSuggestedTitle( const QString& s ) { m_suggestedTitle = s;  }
+#else
+        const QString& suggestedTitle() const { return m_suggestedFilename; }
+#endif
 
         void setShowAnimations( KHTMLSettings::KAnimationAdvice );
 
@@ -290,6 +300,9 @@ namespace khtml
         void do_notify(const QPixmap& p, const QRect& r);
 
         QString m_suggestedFilename;
+#ifdef IMAGE_TITLES
+        QString m_suggestedTitle;
+#endif
 	QMovie* m;
         QPixmap* p;
 	QPixmap* bg;
@@ -309,6 +322,7 @@ namespace khtml
         KHTMLSettings::KAnimationAdvice m_showAnimations : 2;
 
         friend class Cache;
+        friend class ::KHTMLPart;
     };
 
     /**
@@ -347,6 +361,7 @@ namespace khtml
 
         friend class Cache;
         friend class DOM::DocumentImpl;
+        friend class ::KHTMLPart;
 
         QStringList m_reloadedURLs;
         mutable QPtrDict<CachedObject> m_docObjects;
@@ -428,7 +443,7 @@ namespace khtml
 	 * init the cache in case it's not already. This needs to get called once
 	 * before using it.
 	 */
-	static void init();
+	KDE_EXPORT static void init();
 
 	/**
 	 * Ask the cache for some url. Will return a cachedObject, and
@@ -452,19 +467,20 @@ namespace khtml
 	static void setSize( int bytes );
 	static int size() { return maxSize; };
 	static void statistics();
-	static void flush(bool force=false);
+	KDE_EXPORT static void flush(bool force=false);
 
 	/**
 	 * clears the cache
 	 * Warning: call this only at the end of your program, to clean
 	 * up memory (useful for finding memory holes)
 	 */
-	static void clear();
+	KDE_EXPORT static void clear();
 
 	static Loader *loader() { return m_loader; }
 
     	static QPixmap *nullPixmap;
         static QPixmap *brokenPixmap;
+        static QPixmap *blockedPixmap;
         static int cacheSize;
 
         static void removeCacheEntry( CachedObject *object );
