@@ -3,6 +3,7 @@
  *
  * Copyright (C) 2000-2003 Lars Knoll (knoll@kde.org)
  *           (C) 2003-2004 Apple Computer, Inc.
+ *           (C) 2005 Allan Sandfeld Jensen (kde@carewolf.com)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -16,8 +17,8 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 #include "rendering/bidi.h"
@@ -26,6 +27,7 @@
 #include "rendering/render_text.h"
 #include "rendering/render_arena.h"
 #include "rendering/render_layer.h"
+#include "rendering/render_canvas.h"
 #include "xml/dom_docimpl.h"
 
 #include "kdebug.h"
@@ -34,6 +36,7 @@
 
 #define BIDI_DEBUG 0
 //#define DEBUG_LINEBREAKS
+//#define PAGE_DEBUG
 
 namespace khtml {
 
@@ -417,6 +420,19 @@ static void reverseRuns(int start, int end)
         sLastBidiRun = startRun;
 }
 
+static void chopMidpointsAt(RenderObject* obj, uint pos)
+{
+    if (!sNumMidpoints) return;
+    BidiIterator* midpoints = smidpoints->data();
+    for (uint i = 0; i < sNumMidpoints; i++) {
+        const BidiIterator& point = midpoints[i];
+        if (point.obj == obj && point.pos == pos) {
+            sNumMidpoints = i;
+            break;
+        }
+    }
+}
+
 static void checkMidpoints(BidiIterator& lBreak, BidiState &bidi)
 {
     // Check to see if our last midpoint is a start point beyond the line break.  If so,
@@ -432,8 +448,16 @@ static void checkMidpoints(BidiIterator& lBreak, BidiState &bidi)
         if (currpoint == lBreak) {
             // We hit the line break before the start point.  Shave off the start point.
             sNumMidpoints--;
-            if (endpoint.obj->style()->whiteSpace() != PRE)
+            if (!endpoint.obj->style()->preserveWS()) {
+                if (endpoint.obj->isText()) {
+                    // Don't shave a character off the endpoint if it was from a soft hyphen.
+                    RenderText* textObj = static_cast<RenderText*>(endpoint.obj);
+                    if (endpoint.pos+1 < textObj->length() &&
+                        textObj->text()[endpoint.pos+1].unicode() == SOFT_HYPHEN)
+                        return;
+                }
                 endpoint.pos--;
+            }
         }
     }
 }
@@ -580,10 +604,10 @@ static void embed( QChar::Direction d, BidiState &bidi )
 
 	    bidi.context = new BidiContext(level, runDir, bidi.context, override);
 	    bidi.context->ref();
-	    if ( override )
-		dir = runDir;
+	    dir = runDir;
 	    bidi.status.last = runDir;
 	    bidi.status.lastStrong = runDir;
+	    bidi.status.eor = runDir;
 	}
     }
     adjustEmbedding = b;
@@ -734,7 +758,7 @@ void RenderBlock::computeHorizontalPositionsForLine(InlineFlowBox* lineBox, Bidi
                 KHTMLAssert(spaces <= numSpaces);
 
                 // Only justify text with white-space: normal.
-                if (r->obj->style()->whiteSpace() != PRE) {
+                if (r->obj->style()->whiteSpace() == NORMAL) {
                     spaceAdd = (availableWidth - totWidth)*spaces/numSpaces;
                     static_cast<InlineTextBox*>(r->box)->setSpaceAdd(spaceAdd);
                     totWidth += spaceAdd;
@@ -756,6 +780,11 @@ void RenderBlock::computeVerticalPositionsForLine(InlineFlowBox* lineBox)
     lineBox->verticallyAlignBoxes(m_height);
 //    lineBox->setBlockHeight(m_height);
 
+    // Check for page-breaks
+    if (canvas()->pagedMode() && !lineBox->afterPageBreak())
+        // If we get a page-break we might need to redo the line-break
+        if (clearLineOfPageBreaks(lineBox) && hasFloats()) return;
+
     // See if the line spilled out.  If so set overflow height accordingly.
     int bottomOfLine = lineBox->bottomOverflow();
     if (bottomOfLine > m_height && bottomOfLine > m_overflowHeight)
@@ -774,6 +803,76 @@ void RenderBlock::computeVerticalPositionsForLine(InlineFlowBox* lineBox)
     }
 }
 
+bool RenderBlock::clearLineOfPageBreaks(InlineFlowBox* lineBox)
+{
+    bool doPageBreak = false;
+    // Check for physical page-breaks
+    int xpage = crossesPageBreak(lineBox->topOverflow(), lineBox->bottomOverflow());
+    if (xpage) {
+#ifdef PAGE_DEBUG
+        kdDebug(6040) << renderName() << " Line crosses to page " << xpage << endl;
+        kdDebug(6040) << renderName() << " at pos " << lineBox->yPos() << " height " << lineBox->height() << endl;
+#endif
+
+        doPageBreak = true;
+        // check page-break-inside
+        if (!style()->pageBreakInside()) {
+            if (parent()->canClear(this, PageBreakNormal)) {
+                setNeedsPageClear(true);
+                doPageBreak = false;
+            }
+#ifdef PAGE_DEBUG
+            else
+                kdDebug(6040) << "Ignoring page-break-inside: avoid" << endl;
+#endif
+        }
+        // check orphans
+        int orphans = 0;
+        InlineRunBox* box = lineBox->prevLineBox();
+        while (box && orphans < style()->orphans()) {
+            orphans++;
+            box = box->prevLineBox();
+        }
+
+        if (orphans < style()->orphans()) {
+#ifdef PAGE_DEBUG
+            kdDebug(6040) << "Orphans: " << orphans << endl;
+#endif
+            // Orphans is a level 2 page-break rule and can be broken only
+            // if the break is physically required.
+            if (parent()->canClear(this, PageBreakHarder)) {
+                // move block instead
+                setNeedsPageClear(true);
+                doPageBreak = false;
+            }
+#ifdef PAGE_DEBUG
+            else
+                kdDebug(6040) << "Ignoring violated orphans" << endl;
+#endif
+        }
+        if (doPageBreak) {
+            int oldYPos = lineBox->yPos();
+            int pTop = pageTopAfter(lineBox->yPos());
+
+            m_height = pTop;
+            lineBox->setAfterPageBreak(true);
+            lineBox->verticallyAlignBoxes(m_height);
+            if (lineBox->yPos() < pTop) {
+                // ### serious crap. render_line is sometimes placing lines too high
+                kdDebug(6040) << "page top overflow by repositioned line" << endl;
+                int heightIncrease = pTop - lineBox->yPos();
+                m_height = pTop + heightIncrease;
+                lineBox->verticallyAlignBoxes(m_height);
+            }
+#ifdef PAGE_DEBUG
+            kdDebug(6040) << "Cleared line " << lineBox->yPos() - oldYPos << "px" << endl;
+#endif
+            setContainsPageBreak(true);
+        }
+    }
+    return doPageBreak;
+}
+
 // collects one line of the paragraph and transforms it to visual order
 void RenderBlock::bidiReorderLine(const BidiIterator &start, const BidiIterator &end, BidiState &bidi)
 {
@@ -830,7 +929,6 @@ void RenderBlock::bidiReorderLine(const BidiIterator &start, const BidiIterator
         case QChar::DirRLO:
         case QChar::DirLRO:
         case QChar::DirPDF:
-            bidi.eor = bidi.last;
             embed( dirCurrent, bidi );
             break;
 
@@ -1017,7 +1115,7 @@ void RenderBlock::bidiReorderLine(const BidiIterator &start, const BidiIterator
                     break;
                 case QChar::DirCS:
                     if(bidi.status.eor == QChar::DirAN) {
-                        bidi.eor = bidi.current; bidi.status.eor = QChar::DirR; break;
+                        bidi.eor = bidi.current; break;
                     }
                 case QChar::DirES:
                 case QChar::DirET:
@@ -1201,8 +1299,6 @@ void RenderBlock::bidiReorderLine(const BidiIterator &start, const BidiIterator
                 }
                 int end = i-1;
                 reverseRuns(start, end);
-                i++;
-                if(i >= count) break;
             }
             levelHigh--;
         }
@@ -1254,7 +1350,7 @@ static void buildCompactRuns(RenderObject* compactObj, BidiState &bidi)
 }
 #endif
 
-void RenderBlock::layoutInlineChildren(bool relayoutChildren)
+void RenderBlock::layoutInlineChildren(bool relayoutChildren, int breakBeforeLine)
 {
     BidiState bidi;
 
@@ -1282,6 +1378,10 @@ void RenderBlock::layoutInlineChildren(bool relayoutChildren)
         // layout replaced elements
         RenderObject *o = first( this, bidi, false );
         while ( o ) {
+            if (o->markedForRepaint()) {
+                o->repaintDuringLayout();
+                o->setMarkedForRepaint(false);
+             }
             if (o->isReplaced() || o->isFloating() || o->isPositioned()) {
                 // clear the placeHolderBox
                 if (o->isBox())
@@ -1333,10 +1433,24 @@ void RenderBlock::layoutInlineChildren(bool relayoutChildren)
 
         previousLineBrokeAtBR = true;
 
+        int lineCount = 0;
+        bool pagebreakHint = false;
+        int oldPos = 0;
+        BidiIterator oldStart;
+        BidiState oldBidi;
+        const bool pagedMode = canvas()->pagedMode();
+//
         while( !end.atEnd() ) {
             start = end;
+            lineCount++;
             betweenMidpoints = false;
             isLineEmpty = true;
+            pagebreakHint = false;
+            if (pagedMode) {
+                oldPos = m_height;
+                oldStart = start;
+                oldBidi = bidi;
+            }
 #ifdef APPLE_CHANGES    // KDE handles compact blocks differently
             if (m_firstLine && firstChild() && firstChild()->isCompact()) {
                 buildCompactRuns(firstChild(), bidi);
@@ -1344,6 +1458,11 @@ void RenderBlock::layoutInlineChildren(bool relayoutChildren)
                 end = start;
             }
 #endif
+            if (lineCount == breakBeforeLine) {
+                m_height = pageTopAfter(oldPos);
+                pagebreakHint = true;
+            }
+redo_linebreak:
             end = findNextLineBreak(start, bidi);
             if( start.atEnd() ) break;
             if (!isLineEmpty) {
@@ -1352,6 +1471,8 @@ void RenderBlock::layoutInlineChildren(bool relayoutChildren)
                 // Now that the runs have been ordered, we create the line boxes.
                 // At the same time we figure out where border/padding/margin should be applied for
                 // inline flow boxes.
+
+#ifdef APPLE_CHANGES    // KDE handles compact blocks differently
                 if (sCompactFirstBidiRun) {
                     // We have a compact line sharing this line.  Link the compact runs
                     // to our runs to create a single line of runs.
@@ -1359,10 +1480,12 @@ void RenderBlock::layoutInlineChildren(bool relayoutChildren)
                     sFirstBidiRun = sCompactFirstBidiRun;
                     sBidiRunCount += sCompactBidiRunCount;
                 }
-
+#endif
                 if (sBidiRunCount) {
                     InlineFlowBox* lineBox = constructLine(start, end);
                     if (lineBox) {
+                        if (pagebreakHint) lineBox->setAfterPageBreak(true);
+
                         // Now we position all of our text runs horizontally.
                         computeHorizontalPositionsForLine(lineBox, bidi);
 
@@ -1370,6 +1493,15 @@ void RenderBlock::layoutInlineChildren(bool relayoutChildren)
                         computeVerticalPositionsForLine(lineBox);
 
                         deleteBidiRuns(renderArena());
+
+                        if (lineBox->afterPageBreak() && hasFloats() && !pagebreakHint) {
+                            start = end = oldStart;
+                            bidi = oldBidi;
+                            m_height = pageTopAfter(oldPos);
+                            deleteLastLineBox(renderArena());
+                            pagebreakHint = true;
+                            goto redo_linebreak;
+                        }
                     }
                 }
 
@@ -1377,7 +1509,7 @@ void RenderBlock::layoutInlineChildren(bool relayoutChildren)
                     adjustEmbedding = true;
                     end.increment(bidi);
                     adjustEmbedding = false;
-                } else if (end.obj && end.obj->style()->whiteSpace() == PRE && end.current() == QChar('\n')) {
+                } else if (end.obj && end.obj->style()->preserveLF() && end.current() == QChar('\n')) {
                     adjustEmbedding = true;
                     end.increment(bidi);
                     adjustEmbedding = false;
@@ -1399,6 +1531,50 @@ void RenderBlock::layoutInlineChildren(bool relayoutChildren)
     sNumMidpoints = 0;
     sCurrMidpoint = 0;
 
+    // If we violate widows page-breaking rules, we set a hint and relayout.
+    // Note that the widows rule might still be violated afterwards if the lines have become wider
+    if (canvas()->pagedMode() && containsPageBreak() && breakBeforeLine == 0)
+    {
+        int orphans = 0;
+        int widows = 0;
+        // find breaking line
+        InlineRunBox* lineBox = firstLineBox();
+        while (lineBox) {
+            if (lineBox->isInlineFlowBox()) {
+                InlineFlowBox* flowBox = static_cast<InlineFlowBox*>(lineBox);
+                if (flowBox->afterPageBreak()) break;
+            }
+            orphans++;
+            lineBox = lineBox->nextLineBox();
+        }
+        InlineFlowBox* pageBreaker = static_cast<InlineFlowBox*>(lineBox);
+        if (!pageBreaker) goto no_break;
+        // count widows
+        while (lineBox && widows < style()->widows()) {
+            if (lineBox->hasTextChildren())
+                widows++;
+            lineBox = lineBox->nextLineBox();
+        }
+        if (widows < style()->widows()) {
+            kdDebug( 6040 ) << "Widows: " << widows << endl;
+            // Check if we have enough orphans after respecting widows count
+            int newOrphans = orphans - (style()->widows() - widows);
+            if (newOrphans < style()->orphans() && parent()->canClear(this,PageBreakHarder))
+            {
+                // Relayout to remove incorrect page-break
+                setNeedsPageClear(true);
+                setContainsPageBreak(false);
+                layoutInlineChildren(relayoutChildren, -1);
+                return;
+            } else {
+                // Set hint and try again
+                layoutInlineChildren(relayoutChildren, newOrphans+1);
+                return;
+            }
+        }
+    }
+    no_break:
+
     // in case we have a float on the last line, it might not be positioned up to now.
     // This has to be done before adding in the bottom border/padding, or the float will
     // include the padding incorrectly. -dwh
@@ -1418,6 +1594,31 @@ void RenderBlock::layoutInlineChildren(bool relayoutChildren)
     //kdDebug(6040) << "height = " << m_height <<endl;
 }
 
+static void setStaticPosition( RenderBlock* p, RenderObject *o, bool *needToSetStaticX = 0, bool *needToSetStaticY = 0 )
+{
+      // If our original display wasn't an inline type, then we can
+      // determine our static x position now.
+      bool nssx, nssy;
+      bool isInlineType = o->style()->isOriginalDisplayInlineType();
+      nssx = o->hasStaticX();
+      if (nssx && !isInlineType && o->isBox()) {
+          static_cast<RenderBox*>(o)->setStaticX(o->parent()->style()->direction() == LTR ?
+                                  p->borderLeft()+p->paddingLeft() :
+                                  p->borderRight()+p->paddingRight());
+          nssx = false;
+      }
+
+      // If our original display was an INLINE type, then we can
+      // determine our static y position now.
+      nssy = o->hasStaticY();
+      if (nssy && o->isBox()) {
+          static_cast<RenderBox*>(o)->setStaticY(p->height());
+          nssy = !isInlineType;
+      }
+      if (needToSetStaticX) *needToSetStaticX = nssx;
+      if (needToSetStaticY) *needToSetStaticY = nssy;
+}
+
 BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi)
 {
     int width = lineWidth(m_height);
@@ -1428,13 +1629,17 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
     kdDebug(6041) << "sol: " << start.obj << " " << start.pos << endl;
 #endif
 
+    BidiIterator posStart = start;
+    bool hadPosStart = false;
+
     // eliminate spaces at beginning of line
     // remove leading spaces.  Any inline flows we encounter will be empty and should also
     // be skipped.
-    while (!start.atEnd() && (start.obj->isInlineFlow() || (start.obj->style()->whiteSpace() != PRE && !start.obj->isBR() &&
+    while (!start.atEnd() && (start.obj->isInlineFlow() || (!start.obj->style()->preserveWS() && !start.obj->isBR() &&
 #ifndef QT_NO_UNICODETABLES
         ( (start.current().unicode() == (ushort)0x0020) || // ASCII space
           (start.current().unicode() == (ushort)0x0009) || // ASCII tab
+          (start.current().unicode() == (ushort)0x000A) || // ASCII line feed
           (start.current().unicode() == (ushort)0x000C) || // ASCII form feed
           (start.current().unicode() == (ushort)0x200B) || // Zero-width space
           start.obj->isFloatingOrPositioned() )
@@ -1451,21 +1656,33 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
                 width = lineWidth(m_height);
             }
             else if (o->isBox() && o->isPositioned()) {
-                if (o->hasStaticX())
-                    static_cast<RenderBox*>(o)->setStaticX(style()->direction() == LTR ?
-                                  borderLeft()+paddingLeft() :
-                                  borderRight()+paddingRight());
-                if (o->hasStaticY())
-                    static_cast<RenderBox*>(o)->setStaticY(m_height);
+                if (!hadPosStart) {
+                    hadPosStart = true;
+                    posStart = start;
+                    // end
+                    addMidpoint(BidiIterator(0, o, 0));
+                } else {
+                    // start/end
+                    addMidpoint(BidiIterator(0, o, 0));
+                    addMidpoint(BidiIterator(0, o, 0));
+                }
+                setStaticPosition(this, o);
             }
         }
-
         adjustEmbedding = true;
         start.increment(bidi);
         adjustEmbedding = false;
     }
 
+    if (hadPosStart && !start.atEnd())
+        addMidpoint(start);
+
     if ( start.atEnd() ){
+        if (hadPosStart) {
+            start = posStart;
+            posStart.increment(bidi);
+            return posStart;
+        }
         return start;
     }
 
@@ -1497,7 +1714,6 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
             if( w + tmpW <= width ) {
                 lBreak.obj = o;
                 lBreak.pos = 0;
-//                lBreak.increment(bidi);
 
                 // A <br> always breaks a line, so don't let the line be collapsed
                 // away. Also, the space at the end of a line with a <br> does not
@@ -1531,24 +1747,9 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
                 }
             }
             else if (o->isPositioned()) {
-                // If our original display wasn't an inline type, then we can
-                // go ahead and determine our static x position now.
-                bool isInlineType = o->style()->isOriginalDisplayInlineType();
-                bool needToSetStaticX = o->hasStaticX();
-                if (o->hasStaticX() && !isInlineType && o->isBox()) {
-                    static_cast<RenderBox*>(o)->setStaticX(o->parent()->style()->direction() == LTR ?
-                                  borderLeft()+paddingLeft() :
-                                  borderRight()+paddingRight());
-                    needToSetStaticX = false;
-                }
-
-                // If our original display was an INLINE type, then we can go ahead
-                // and determine our static y position now.
-                bool needToSetStaticY = o->hasStaticY();
-                if (o->hasStaticY() && isInlineType && o->isBox()) {
-                    static_cast<RenderBox*>(o)->setStaticY(m_height);
-                    needToSetStaticY = false;
-                }
+                bool needToSetStaticX;
+                bool needToSetStaticY;
+                setStaticPosition(this, o, &needToSetStaticX, &needToSetStaticY);
 
                 // If we're ignoring spaces, we have to stop and include this object and
                 // then start ignoring spaces again.
@@ -1581,9 +1782,6 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
                 lastWS = last->layer()->marquee()->whiteSpace();
 
             // Break on replaced elements if either has normal white-space.
-            // FIXME: This does not match WinIE, Opera, and Mozilla.  They treat replaced elements
-            // like characters in a word, and require spaces between the replaced elements in order
-            // to break.
             if (currWS == NORMAL || lastWS == NORMAL) {
                 w += tmpW;
                 tmpW = 0;
@@ -1627,26 +1825,56 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
             const Font *f = t->htmlFont( m_firstLine );
             // proportional font, needs a bit more work.
             int lastSpace = pos;
-            bool isPre = o->style()->whiteSpace() == PRE;
+            bool autoWrap = o->style()->autoWrap();
+            bool preserveWS = o->style()->preserveWS();
+            bool preserveLF = o->style()->preserveLF();
 #ifdef APPLE_CHANGES
             int wordSpacing = o->style()->wordSpacing();
 #endif
             bool appliedStartWidth = pos > 0; // If the span originated on a previous line,
                                               // then assume the start width has been applied.
             bool appliedEndWidth = false;
+            bool nextIsSoftBreakable = false;
 
             while(len) {
                 bool previousCharacterIsSpace = currentCharacterIsSpace;
+                bool isSoftBreakable = nextIsSoftBreakable;
+                nextIsSoftBreakable = false;
                 const QChar c = str[pos];
-                currentCharacterIsSpace = c == ' ' || (!isPre && c == '\n');
+                currentCharacterIsSpace = c == ' ';
 
-                if (isPre || !currentCharacterIsSpace)
+                if (preserveWS || !currentCharacterIsSpace)
                     isLineEmpty = false;
 
+                // Check for soft hyphens.  Go ahead and ignore them.
+                if (c.unicode() == SOFT_HYPHEN && pos > 0) {
+                    nextIsSoftBreakable = true;
+                    if (!ignoringSpaces) {
+                        // Ignore soft hyphens
+                        BidiIterator endMid(0, o, pos-1);
+                        addMidpoint(endMid);
+
+                        // Add the width up to but not including the hyphen.
+                        tmpW += t->width(lastSpace, pos - lastSpace, f);
+
+                        // For whitespace normal only, include the hyphen.  We need to ensure it will fit
+                        // on the line if it shows when we break.
+                        if (o->style()->whiteSpace() == NORMAL)
+                            tmpW += t->width(pos, 1, f);
+
+                        BidiIterator startMid(0, o, pos+1);
+                        addMidpoint(startMid);
+                    }
+
+                    pos++;
+                    len--;
+                    lastSpace = pos; // Cheesy hack to prevent adding in widths of the run twice.
+                    continue;
+                }
 #ifdef APPLE_CHANGES    // KDE applies wordspacing differently
                 bool applyWordSpacing = false;
 #endif
-                if ( (isPre && c == '\n') || (!isPre && isBreakable( str, pos, strlen )) ) {
+                if ( (preserveLF && c == '\n') || (autoWrap && (isBreakable( str, pos, strlen ) || isSoftBreakable)) ) {
                     if (ignoringSpaces) {
                         if (!currentCharacterIsSpace) {
                             // Stop ignoring spaces and begin at this
@@ -1676,7 +1904,7 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
 #ifdef DEBUG_LINEBREAKS
                     kdDebug(6041) << "found space at " << pos << " in string '" << QString( str, strlen ).latin1() << "' adding " << tmpW << " new width = " << w << endl;
 #endif
-                    if ( !isPre && w + tmpW > width && w == 0 ) {
+                    if ( autoWrap && w + tmpW > width && w == 0 ) {
                         int fb = nearestFloatBottom(m_height);
                         int newLineWidth = lineWidth(fb);
                         // See if |tmpW| will fit on the new line.  As long as it does not,
@@ -1697,11 +1925,15 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
                         }
                     }
 
-                    if (w + tmpW > width && o->style()->whiteSpace() == NORMAL) {
-                        goto end;
+                    if (autoWrap) {
+                        if (w+tmpW > width)
+                            goto end;
+                        else if ( (pos > 1 && str[pos-1].unicode() == SOFT_HYPHEN) )
+                            // Subtract the width of the soft hyphen out since we fit on a line.
+                            tmpW -= t->width(pos-1, 1, f);
                     }
 
-                    if( *(str+pos) == '\n' && isPre) {
+                    if( preserveLF && *(str+pos) == '\n' ) {
                         lBreak.obj = o;
                         lBreak.pos = pos;
 
@@ -1711,7 +1943,7 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
                         return lBreak;
                     }
 
-                    if (o->style()->whiteSpace() == NORMAL) {
+                    if ( autoWrap ) {
                         w += tmpW;
                         tmpW = 0;
                         lBreak.obj = o;
@@ -1723,7 +1955,7 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
                     if (applyWordSpacing)
                         w += wordSpacing;
 #endif
-                    if (!ignoringSpaces && !isPre) {
+                    if (!ignoringSpaces && !preserveWS) {
                         // If we encounter a newline, or if we encounter a
                         // second space, we need to go ahead and break up this
                         // run and enter a mode where we start collapsing spaces.
@@ -1752,9 +1984,9 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
                     ignoreStart.pos = pos;
                 }
 
-                if (!isPre && currentCharacterIsSpace && !ignoringSpaces)
+                if (!preserveWS && currentCharacterIsSpace && !ignoringSpaces)
                     trailingSpaceObject = o;
-                else if (isPre || !currentCharacterIsSpace)
+                else if (preserveWS || !currentCharacterIsSpace)
                     trailingSpaceObject = 0;
 
                 pos++;
@@ -1772,21 +2004,19 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
             KHTMLAssert( false );
 
         RenderObject* next = Bidinext(start.par, o, bidi);
-        bool isNormal = o->style()->whiteSpace() == NORMAL;
+        bool isNormal = o->style()->autoWrap();
         bool checkForBreak = isNormal;
-        if (w && w + tmpW > width+1 && lBreak.obj && o->style()->whiteSpace() == NOWRAP)
+        if (w && w + tmpW > width+1 && lBreak.obj && !o->style()->preserveLF())
             checkForBreak = true;
         else if (next && o->isText() && next->isText() && !next->isBR()) {
-            if (isNormal || (next->style()->whiteSpace() == NORMAL)) {
+            if (isNormal || next->style()->autoWrap()) {
                 if (currentCharacterIsSpace)
                     checkForBreak = true;
                 else {
                     RenderText* nextText = static_cast<RenderText*>(next);
                     int strlen = nextText->stringLength();
                     QChar *str = nextText->text();
-                    if (strlen &&
-                        ((str[0].unicode() == ' ') ||
-                            (next->style()->whiteSpace() != PRE && str[0] == '\n')))
+                    if (strlen && str[0] == ' ')
                         // If the next item on the line is text, and if we did not end with
                         // a space, then the next text run continues our word (and so it needs to
                         // keep adding to |tmpW|.  Just update and continue.
@@ -1809,7 +2039,7 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
             //kdDebug() << " too wide w=" << w << " tmpW = " << tmpW << " width = " << width << endl;
             //kdDebug() << "start=" << start.obj << " current=" << o << endl;
             // if we have floats, try to get below them.
-            if (currentCharacterIsSpace && !ignoringSpaces && o->style()->whiteSpace() != PRE)
+            if (currentCharacterIsSpace && !ignoringSpaces && !o->style()->preserveWS())
                 trailingSpaceObject = 0;
 
             int fb = nearestFloatBottom(m_height);
@@ -1840,7 +2070,7 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
         last = o;
         o = next;
 
-        if (!last->isFloatingOrPositioned() && last->isReplaced() && last->style()->whiteSpace() == NORMAL) {
+        if (!last->isFloatingOrPositioned() && last->isReplaced() && last->style()->autoWrap()) {
             // Go ahead and add in tmpW.
             w += tmpW;
             tmpW = 0;
@@ -1850,7 +2080,7 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
 
         // Clear out our character space bool, since inline <pre>s don't collapse whitespace
         // with adjacent inline normal/nowrap spans.
-        if (last->style()->whiteSpace() == PRE)
+        if (last->style()->preserveWS())
             currentCharacterIsSpace = false;
 
         pos = 0;
@@ -1859,7 +2089,7 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
 #ifdef DEBUG_LINEBREAKS
     kdDebug( 6041 ) << "end of par, width = " << width << " linewidth = " << w + tmpW << endl;
 #endif
-    if( w + tmpW <= width || (last && last->style()->whiteSpace() == NOWRAP)) {
+    if( w + tmpW <= width || (last && !last->style()->autoWrap())) {
         lBreak.obj = 0;
         lBreak.pos = 0;
     }
@@ -1891,6 +2121,9 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
         }
     }
 
+    if (hadPosStart)
+        start = posStart;
+
     // make sure we consume at least one char/object.
     if( lBreak == start )
         lBreak.increment(bidi);
@@ -1930,6 +2163,14 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
         lBreak.increment(bidi);
     }
 
+    if (lBreak.obj && lBreak.pos >= 2 && lBreak.obj->isText()) {
+        // For soft hyphens on line breaks, we have to chop out the midpoints that made us
+        // ignore the hyphen so that it will render at the end of the line.
+        QChar c = static_cast<RenderText*>(lBreak.obj)->text()[lBreak.pos-1];
+        if (c.unicode() == SOFT_HYPHEN)
+            chopMidpointsAt(lBreak.obj, lBreak.pos-2);
+    }
+
     return lBreak;
 }
 
