@@ -18,8 +18,8 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 //#define DEBUG_LAYOUT
@@ -662,8 +662,16 @@ void RenderPartObject::updateWidget()
           params.append( QString::fromLatin1("__KHTML__CODEBASE=\"%1\"").arg( objbase->getAttribute(ATTR_CODEBASE).string() ) );
           if (!objbase->getAttribute(ATTR_WIDTH).isEmpty())
               params.append( QString::fromLatin1("WIDTH=\"%1\"").arg( objbase->getAttribute(ATTR_WIDTH).string() ) );
+          else if (embed && !embed->getAttribute(ATTR_WIDTH).isEmpty()) {
+              params.append( QString::fromLatin1("WIDTH=\"%1\"").arg( embed->getAttribute(ATTR_WIDTH).string() ) );
+              objbase->setAttribute(ATTR_WIDTH, embed->getAttribute(ATTR_WIDTH));
+          }
           if (!objbase->getAttribute(ATTR_HEIGHT).isEmpty())
               params.append( QString::fromLatin1("HEIGHT=\"%1\"").arg( objbase->getAttribute(ATTR_HEIGHT).string() ) );
+          else if (embed && !embed->getAttribute(ATTR_HEIGHT).isEmpty()) {
+              params.append( QString::fromLatin1("HEIGHT=\"%1\"").arg( embed->getAttribute(ATTR_HEIGHT).string() ) );
+              objbase->setAttribute(ATTR_HEIGHT, embed->getAttribute(ATTR_HEIGHT));
+          }
 
           if ( embed ) {
               // render embed object
@@ -813,7 +821,7 @@ void RenderPartObject::slotPartLoadingErrorNotify()
             QString shortURL = pluginPageURL.protocol() == "http" ? pluginPageURL.host() : pluginPageURL.prettyURL();
             int res = KMessageBox::questionYesNo( m_view,
                                                   i18n("No plugin found for '%1'.\nDo you want to download one from %2?").arg(mimeName).arg(shortURL),
-                                                  i18n("Missing Plugin"), QString::null, QString::null, QString("plugin-")+serviceType);
+                                                  i18n("Missing Plugin"), i18n("Download"), i18n("Do Not Download"), QString("plugin-")+serviceType);
             if ( res == KMessageBox::Yes )
             {
                 // Display vendor download page
