@@ -24,7 +24,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  * Alternatively, the contents of this file may be used under the terms
  * of either the Mozilla Public License Version 1.1, found at
@@ -200,12 +200,11 @@ public:
     void paintScrollbars(RenderObject::PaintInfo& pI);
     void checkScrollbarsAfterLayout();
     void slotValueChanged(int);
-    void markForRepaint( bool afterLayout = false );
+    void repaint( bool markForRepaint = false );
     void updateScrollPositionFromScrollbars();
 
     void updateLayerPosition();
-    void updateLayerPositions(RenderLayer* rootLayer, bool doFullRepaint, bool checkForRepaint = false);
-    void layout();
+    void updateLayerPositions( RenderLayer* rootLayer, bool doFullRepaint = false, bool checkForRepaint = false);
 
     // Get the enclosing stacking context for this layer.  A stacking context is a layer
     // that has a non-auto z-index.
@@ -230,7 +229,7 @@ public:
     // paints the layers that intersect the damage rect from back to
     // front.  The nodeAtPoint method looks for mouse events by walking
     // layers that intersect the point from front to back.
-    void paint(QPainter *p, const QRect& damageRect, bool selectionOnly=false);
+    KDE_EXPORT void paint(QPainter *p, const QRect& damageRect, bool selectionOnly=false);
     bool nodeAtPoint(RenderObject::NodeInfo& info, int x, int y);
 
     // This method figures out our layerBounds in coordinates relative to
@@ -249,7 +248,7 @@ public:
     void detach(RenderArena* renderArena);
 
 #ifdef ENABLE_DUMP
-    void dump(QTextStream &stream, const QString &ind = QString::null);
+    KDE_EXPORT void dump(QTextStream &stream, const QString &ind = QString::null);
 #endif
 
      // Overloaded new operator.  Derived classes must override operator new
@@ -272,7 +271,7 @@ private:
 
     void collectLayers(QPtrVector<RenderLayer>*&, QPtrVector<RenderLayer>*&);
 
-    void paintLayer(RenderLayer* rootLayer, QPainter *p, const QRect& paintDirtyRect, bool selectionOnly=false);
+    KDE_EXPORT void paintLayer(RenderLayer* rootLayer, QPainter *p, const QRect& paintDirtyRect, bool selectionOnly=false);
     RenderLayer* nodeAtPointForLayer(RenderLayer* rootLayer, RenderObject::NodeInfo& info,
                                      int x, int y, const QRect& hitTestRect);
 
