@@ -4,7 +4,8 @@
  * Copyright (C) 1999 Antti Koivisto (koivisto@kde.org)
  * Copyright (C) 1999-2003 Lars Knoll (knoll@kde.org)
  * Copyright (C) 2002-2003 Dirk Mueller (mueller@kde.org)
- * Copyright (C) 2002-2003 Apple Computer, Inc.
+ * Copyright (C) 2002-2004 Apple Computer, Inc.
+ * Copyright (C) 2005 Allan Sandfeld Jensen (kde@carewolf.com)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -18,8 +19,8 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 
@@ -102,29 +103,148 @@ StyleVisualData::StyleVisualData(const StyleVisualData& o )
 {
 }
 
+BackgroundLayer::BackgroundLayer()
+:m_image(RenderStyle::initialBackgroundImage()),
+ m_bgAttachment(RenderStyle::initialBackgroundAttachment()),
+ m_bgRepeat(RenderStyle::initialBackgroundRepeat()),
+ m_next(0)
+{
+    m_imageSet = m_attachmentSet = m_repeatSet = m_xPosSet = m_yPosSet = false;
+}
 
+BackgroundLayer::BackgroundLayer(const BackgroundLayer& o)
+{
+    m_next = o.m_next ? new BackgroundLayer(*o.m_next) : 0;
+    m_image = o.m_image;
+    m_xPosition = o.m_xPosition;
+    m_yPosition = o.m_yPosition;
+    m_bgAttachment = o.m_bgAttachment;
+    m_bgRepeat = o.m_bgRepeat;
+    m_imageSet = o.m_imageSet;
+    m_attachmentSet = o.m_attachmentSet;
+    m_repeatSet = o.m_repeatSet;
+    m_xPosSet = o.m_xPosSet;
+    m_yPosSet = o.m_yPosSet;
+}
+
+BackgroundLayer::~BackgroundLayer()
+{
+    delete m_next;
+}
+
+BackgroundLayer& BackgroundLayer::operator=(const BackgroundLayer& o) {
+    if (m_next != o.m_next) {
+        delete m_next;
+        m_next = o.m_next ? new BackgroundLayer(*o.m_next) : 0;
+    }
+
+    m_image = o.m_image;
+    m_xPosition = o.m_xPosition;
+    m_yPosition = o.m_yPosition;
+    m_bgAttachment = o.m_bgAttachment;
+    m_bgRepeat = o.m_bgRepeat;
+
+    m_imageSet = o.m_imageSet;
+    m_attachmentSet = o.m_attachmentSet;
+    m_repeatSet = o.m_repeatSet;
+    m_xPosSet = o.m_xPosSet;
+    m_yPosSet = o.m_yPosSet;
+
+    return *this;
+}
+
+bool BackgroundLayer::operator==(const BackgroundLayer& o) const {
+    return m_image == o.m_image && m_xPosition == o.m_xPosition && m_yPosition == o.m_yPosition &&
+           m_bgAttachment == o.m_bgAttachment && m_bgRepeat == o.m_bgRepeat &&
+           m_imageSet == o.m_imageSet && m_attachmentSet == o.m_attachmentSet && m_repeatSet == o.m_repeatSet &&
+           m_xPosSet == o.m_xPosSet && m_yPosSet == o.m_yPosSet &&
+           ((m_next && o.m_next) ? *m_next == *o.m_next : m_next == o.m_next);
+}
+
+void BackgroundLayer::fillUnsetProperties()
+{
+    BackgroundLayer* curr;
+    for (curr = this; curr && curr->isBackgroundImageSet(); curr = curr->next());
+    if (curr && curr != this) {
+        // We need to fill in the remaining values with the pattern specified.
+        for (BackgroundLayer* pattern = this; curr; curr = curr->next()) {
+            curr->m_image = pattern->m_image;
+            pattern = pattern->next();
+            if (pattern == curr || !pattern)
+                pattern = this;
+        }
+    }
+
+    for (curr = this; curr && curr->isBackgroundXPositionSet(); curr = curr->next());
+    if (curr && curr != this) {
+        // We need to fill in the remaining values with the pattern specified.
+        for (BackgroundLayer* pattern = this; curr; curr = curr->next()) {
+            curr->m_xPosition = pattern->m_xPosition;
+            pattern = pattern->next();
+            if (pattern == curr || !pattern)
+                pattern = this;
+        }
+    }
+
+    for (curr = this; curr && curr->isBackgroundYPositionSet(); curr = curr->next());
+    if (curr && curr != this) {
+        // We need to fill in the remaining values with the pattern specified.
+        for (BackgroundLayer* pattern = this; curr; curr = curr->next()) {
+            curr->m_yPosition = pattern->m_yPosition;
+            pattern = pattern->next();
+            if (pattern == curr || !pattern)
+                pattern = this;
+        }
+    }
+
+    for (curr = this; curr && curr->isBackgroundAttachmentSet(); curr = curr->next());
+    if (curr && curr != this) {
+        // We need to fill in the remaining values with the pattern specified.
+        for (BackgroundLayer* pattern = this; curr; curr = curr->next()) {
+            curr->m_bgAttachment = pattern->m_bgAttachment;
+            pattern = pattern->next();
+            if (pattern == curr || !pattern)
+                pattern = this;
+        }
+    }
+
+    for (curr = this; curr && curr->isBackgroundRepeatSet(); curr = curr->next());
+    if (curr && curr != this) {
+        // We need to fill in the remaining values with the pattern specified.
+        for (BackgroundLayer* pattern = this; curr; curr = curr->next()) {
+            curr->m_bgRepeat = pattern->m_bgRepeat;
+            pattern = pattern->next();
+            if (pattern == curr || !pattern)
+                pattern = this;
+        }
+    }
+}
+
+void BackgroundLayer::cullEmptyLayers()
+{
+    BackgroundLayer *next;
+    for (BackgroundLayer *p = this; p; p = next) {
+        next = p->m_next;
+        if (next && !next->isBackgroundImageSet() &&
+            !next->isBackgroundXPositionSet() && !next->isBackgroundYPositionSet() &&
+            !next->isBackgroundAttachmentSet() && !next->isBackgroundRepeatSet()) {
+            delete next;
+            p->m_next = 0;
+            break;
+        }
+    }
+}
 
 StyleBackgroundData::StyleBackgroundData()
-    : image( RenderStyle::initialBackgroundImage() )
-{
-}
+{}
 
-StyleBackgroundData::StyleBackgroundData(const StyleBackgroundData& o )
-    : Shared<StyleBackgroundData>(),
-      color( o.color ), image( o.image ),
-      x_position( o.x_position ), y_position( o.y_position ),
-      outline( o.outline )
-{
-}
+StyleBackgroundData::StyleBackgroundData(const StyleBackgroundData& o)
+    : Shared<StyleBackgroundData>(), m_background(o.m_background), m_outline(o.m_outline)
+{}
 
 bool StyleBackgroundData::operator==(const StyleBackgroundData& o) const
 {
-    return
-	color == o.color &&
-	image == o.image &&
-	x_position == o.x_position &&
-	y_position == o.y_position &&
-	outline == o.outline;
+    return m_background == o.m_background && m_color == o.m_color && m_outline == o.m_outline;
 }
 
 StyleMarqueeData::StyleMarqueeData()
@@ -221,9 +341,8 @@ StyleInheritedData::StyleInheritedData()
       font(), color( RenderStyle::initialColor() ),
       border_hspacing( RenderStyle::initialBorderHorizontalSpacing() ),
       border_vspacing( RenderStyle::initialBorderVerticalSpacing() ),
-      quotes(0),
       widows( RenderStyle::initialWidows() ), orphans( RenderStyle::initialOrphans() ),
-      page_break_inside( RenderStyle::initialPageBreak() )
+      quotes(0)
 {
 }
 
@@ -235,10 +354,10 @@ StyleInheritedData::~StyleInheritedData()
 StyleInheritedData::StyleInheritedData(const StyleInheritedData& o )
     : Shared<StyleInheritedData>(),
       indent( o.indent ), line_height( o.line_height ), style_image( o.style_image ),
-      font( o.font ), color( o.color ), decoration_color( o.decoration_color ),
+      font( o.font ), color( o.color ),
       border_hspacing( o.border_hspacing ),
       border_vspacing( o.border_vspacing ),
-      widows(o.widows), orphans(o.orphans), page_break_inside(o.page_break_inside)
+      widows(o.widows), orphans(o.orphans)
 {
     quotes = o.quotes;
     if (quotes) quotes->ref();
@@ -254,13 +373,11 @@ bool StyleInheritedData::operator==(const StyleInheritedData& o) const
 	style_image == o.style_image &&
 	font == o.font &&
 	color == o.color &&
-	decoration_color == o.decoration_color &&
         border_hspacing == o.border_hspacing &&
         border_vspacing == o.border_vspacing &&
         quotes == o.quotes &&
         widows == o.widows &&
-        orphans == o.orphans &&
-        page_break_inside == o.page_break_inside;
+        orphans == o.orphans ;
 
     // doesn't work because structs are not packed
     //return memcmp(this, &o, sizeof(*this))==0;
@@ -531,16 +648,11 @@ RenderStyle::Diff RenderStyle::diff( const RenderStyle *other ) const
     // Visible:
 // 	EVisibility _visibility : 2;
 //     EOverflow _overflow : 4 ;
-//     EBackgroundRepeat _bg_repeat : 2;
-//     bool _bg_attachment : 1;
 // 	int _text_decorations : 4;
 //     DataRef<StyleBackgroundData> background;
     if (inherited->color != other->inherited->color ||
-        inherited->decoration_color != other->inherited->decoration_color ||
         !(inherited_flags.f._visibility == other->inherited_flags.f._visibility) ||
         !(noninherited_flags.f._overflow == other->noninherited_flags.f._overflow) ||
-        !(noninherited_flags.f._bg_repeat == other->noninherited_flags.f._bg_repeat) ||
-        !(noninherited_flags.f._bg_attachment == other->noninherited_flags.f._bg_attachment) ||
         !(inherited_flags.f._text_decorations == other->inherited_flags.f._text_decorations) ||
         !(noninherited_flags.f._hasClip == other->noninherited_flags.f._hasClip) ||
         visual->textDecoration != other->visual->textDecoration ||
@@ -567,6 +679,17 @@ void RenderStyle::setPaletteColor(QPalette::ColorGroup g, QColorGroup::ColorRole
     visual.access()->palette.setColor(g,r,c);
 }
 
+void RenderStyle::adjustBackgroundLayers()
+{
+    if (backgroundLayers()->next()) {
+        // First we cull out layers that have no properties set.
+        accessBackgroundLayers()->cullEmptyLayers();
+
+        // Next we repeat patterns into layers that don't have some properties set.
+        accessBackgroundLayers()->fillUnsetProperties();
+    }
+}
+
 void RenderStyle::setClip( Length top, Length right, Length bottom, Length left )
 {
     StyleVisualData *data = visual.access();
@@ -668,10 +791,7 @@ void RenderStyle::setContent(DOM::DOMStringImpl* s, bool add)
         lastContent = lastContent->_nextContent;
 
     bool reuseContent = !add;
-    if (add) {
-        if (!lastContent)
-            return; // Something's wrong.  We had no previous content, and we should have.
-
+    if (add && lastContent) {
         if (lastContent->_contentType == CONTENT_TEXT) {
             // We can augment the existing string and share this ContentData node.
             DOMStringImpl* oldStr = lastContent->_content.text;
@@ -731,6 +851,33 @@ void RenderStyle::setContent(DOM::CounterImpl* c, bool add)
     newContentData->_contentType = CONTENT_COUNTER;
 }
 
+void RenderStyle::setContent(EQuoteContent q, bool add)
+{
+    if (q == NO_QUOTE)
+        return;
+
+    ContentData* lastContent = content;
+    while (lastContent && lastContent->_nextContent)
+        lastContent = lastContent->_nextContent;
+
+    bool reuseContent = !add;
+    ContentData* newContentData = 0;
+    if (reuseContent && content) {
+        content->clearContent();
+        newContentData = content;
+    }
+    else
+        newContentData = new ContentData;
+
+    if (lastContent && !reuseContent)
+        lastContent->_nextContent = newContentData;
+    else
+        content = newContentData;
+
+    newContentData->_content.quote = q;
+    newContentData->_contentType = CONTENT_QUOTE;
+}
+
 ContentData::~ContentData()
 {
     clearContent();
@@ -865,26 +1012,6 @@ void RenderStyle::setCounterIncrement(CSSValueListImpl *l)
     if (t) t->deref();
 }
 
-void RenderStyle::addCounterReset(CounterActImpl *c)
-{
-    if (!counter_reset) {
-        counter_reset = new CSSValueListImpl;
-        counter_reset->ref();
-    }
-
-    counter_reset->append(c);
-}
-
-void RenderStyle::addCounterIncrement(CounterActImpl *c)
-{
-    if (!counter_increment) {
-        counter_increment = new CSSValueListImpl;
-        counter_increment->ref();
-    }
-
-    counter_increment->append(c);
-}
-
 #ifdef ENABLE_DUMP
 
 static QString describeFont( const QFont &f)
@@ -921,5 +1048,54 @@ QString RenderStyle::createDiff( const RenderStyle &parent ) const
 
     return res;
 }
-
 #endif
+
+RenderPageStyle::RenderPageStyle() : next(0), m_pageType(ANY_PAGE)
+{
+}
+
+RenderPageStyle::~RenderPageStyle()
+{
+    delete next;
+}
+
+RenderPageStyle* RenderPageStyle::getPageStyle(PageType type)
+{
+    RenderPageStyle *ps = 0;
+    for (ps = this; ps; ps = ps->next)
+        if (ps->m_pageType==type)
+            break;
+    return ps;
+}
+
+RenderPageStyle* RenderPageStyle::addPageStyle(PageType type)
+{
+    RenderPageStyle *ps = getPageStyle(type);
+
+    if (!ps)
+    {
+        ps = new RenderPageStyle(*this); // use the real copy constructor to get an identical copy
+        ps->m_pageType = type;
+
+        ps->next = next;
+        next = ps;
+    }
+
+    return ps;
+}
+
+void RenderPageStyle::removePageStyle(PageType type)
+{
+    RenderPageStyle *ps = next;
+    RenderPageStyle *prev = this;
+
+    while (ps) {
+        if (ps->m_pageType==type) {
+            prev->next = ps->next;
+            delete ps;
+            return;
+        }
+        prev = ps;
+        ps = ps->next;
+    }
+}
