@@ -15,19 +15,21 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
- * $Id: dom2_viewsimpl.cpp,v 1.2 2001/06/02 04:18:00 pmk Exp $
  */
 
 #include "dom2_viewsimpl.h"
+#include "dom_elementimpl.h"
 #include "dom_docimpl.h"
+#include "css/css_renderstyledeclarationimpl.h"
+#include "css/cssproperties.h"
+#include "css/css_stylesheetimpl.h"
 
+using namespace khtml;
 using namespace DOM;
 
-
-
 AbstractViewImpl::AbstractViewImpl(DocumentImpl *_document)
 {
     m_document = _document;
@@ -37,8 +39,14 @@ AbstractViewImpl::~AbstractViewImpl()
 {
 }
 
-CSSStyleDeclarationImpl *AbstractViewImpl::getComputedStyle(ElementImpl */*elt*/, DOMStringImpl */*pseudoElt*/)
+CSSStyleDeclarationImpl *AbstractViewImpl::getComputedStyle(ElementImpl* elt, DOMStringImpl* /*pseudoElt*/)
 {
-    return 0; // ###
+    if (!elt || !elt->renderer())
+        return 0;
+
+    CSSStyleDeclarationImpl* style = new RenderStyleDeclarationImpl( elt );
+    style->setParent( elt->getDocument()->elementSheet() );
+
+    return style;
 }
 
