@@ -4,6 +4,7 @@
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *           (C) 2001 Dirk Mueller (mueller@kde.org)
+ *           (C) 2003 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -17,8 +18,8 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include "dom/dom_exception.h"
@@ -61,6 +62,7 @@ NodeImpl::NodeImpl(DocumentPtr *doc)
       m_hasId( false ),
       m_hasStyle( false ),
       m_attached(false),
+      m_closed(false),
       m_changed( false ),
       m_hasChangedChild( false ),
       m_inDocument( false ),
@@ -323,14 +325,17 @@ void NodeImpl::addEventListener(int id, EventListener *listener, const bool useC
 	m_regdListeners->setAutoDelete(true);
     }
 
-    listener->ref();
-
-    // remove existing identical listener set with identical arguments - the DOM2
-    // spec says that "duplicate instances are discarded" in this case.
-    removeEventListener(id,listener,useCapture);
+    // if this id/listener/useCapture combination is already registered, do nothing.
+    // the DOM2 spec says that "duplicate instances are discarded", and this keeps
+    // the listener order intact.
+    QPtrListIterator<RegisteredEventListener> it(*m_regdListeners);
+    for (; it.current(); ++it)
+        if (*(it.current()) == *rl) {
+            delete rl;
+            return;
+        }
 
     m_regdListeners->append(rl);
-    listener->deref();
 }
 
 void NodeImpl::removeEventListener(int id, EventListener *listener, bool useCapture)
@@ -348,31 +353,38 @@ void NodeImpl::removeEventListener(int id, EventListener *listener, bool useCapt
         }
 }
 
-void NodeImpl::removeHTMLEventListener(int id)
-{
-    if (!m_regdListeners) // nothing to remove
-        return;
-
-    QPtrListIterator<RegisteredEventListener> it(*m_regdListeners);
-    for (; it.current(); ++it)
-        if (it.current()->id == id &&
-            it.current()->listener->eventListenerType() == "_khtml_HTMLEventListener") {
-            m_regdListeners->removeRef(it.current());
-            return;
-        }
-}
-
 void NodeImpl::setHTMLEventListener(int id, EventListener *listener)
 {
-    // in case we already have it, we don't want removeHTMLEventListener to destroy it
-    if (listener)
-        listener->ref();
-    removeHTMLEventListener(id);
-    if (listener)
-    {
-        addEventListener(id,listener,false);
-        listener->deref();
+    if (!m_regdListeners) {
+        m_regdListeners = new QPtrList<RegisteredEventListener>;
+        m_regdListeners->setAutoDelete(true);
     }
+
+    QPtrListIterator<RegisteredEventListener> it(*m_regdListeners);
+
+    if (!listener) {
+        for (; it.current(); ++it)
+            if (it.current()->id == id &&
+                it.current()->listener->eventListenerType() == "_khtml_HTMLEventListener") {
+                m_regdListeners->removeRef(it.current());
+                break;
+            }
+            return;
+    }
+
+    // if this event already has a registered handler, insert the new one in
+    // place of the old one, to preserve the order.
+    RegisteredEventListener *rl = new RegisteredEventListener(static_cast<EventImpl::EventId>(id),listener,false);
+
+    for (int i = 0; it.current(); ++it, ++i)
+        if (it.current()->id == id &&
+            it.current()->listener->eventListenerType() == "_khtml_HTMLEventListener") {
+            // Qt4: don't forget to delete the old one first
+            m_regdListeners->replace(i, rl);
+            return;
+        }
+
+    m_regdListeners->append(rl);
 }
 
 EventListener *NodeImpl::getHTMLEventListener(int id)
@@ -652,14 +664,16 @@ void NodeImpl::handleLocalEvents(EventImpl *evt, bool useCapture)
         // ECMA legacy hack
         if (current->useCapture == useCapture && evt->id() == EventImpl::CLICK_EVENT) {
             MouseEventImpl* me = static_cast<MouseEventImpl*>(evt);
-            // To find whether to call onclick or ondblclick, we can't
-            // * use me->detail(), it's 2 when clicking twice w/o moving, even very slowly
-            // * use me->qEvent(), it's not available when using initMouseEvent/dispatchEvent
-            // So we currently store a bool in MouseEventImpl. If anyone needs to trigger
-            // dblclicks from the DOM API, we'll need a timer here (well in the doc).
-            if ( ( !me->isDoubleClick() && current->id == EventImpl::KHTML_ECMA_CLICK_EVENT) ||
-              ( me->isDoubleClick() && current->id == EventImpl::KHTML_ECMA_DBLCLICK_EVENT) )
-                current->listener->handleEvent(ev);
+            if (me->button() == 0) {
+                // To find whether to call onclick or ondblclick, we can't
+                // * use me->detail(), it's 2 when clicking twice w/o moving, even very slowly
+                // * use me->qEvent(), it's not available when using initMouseEvent/dispatchEvent
+                // So we currently store a bool in MouseEventImpl. If anyone needs to trigger
+                // dblclicks from the DOM API, we'll need a timer here (well in the doc).
+                if ( ( !me->isDoubleClick() && current->id == EventImpl::KHTML_ECMA_CLICK_EVENT) ||
+                  ( me->isDoubleClick() && current->id == EventImpl::KHTML_ECMA_DBLCLICK_EVENT) )
+                    current->listener->handleEvent(ev);
+            }
         }
     }
 }
@@ -680,7 +694,7 @@ NodeImpl *NodeImpl::childNode(unsigned long /*index*/)
 
 NodeImpl *NodeImpl::traverseNextNode(NodeImpl *stayWithin) const
 {
-    if (firstChild())
+    if (firstChild() || stayWithin == this)
 	return firstChild();
     else if (nextSibling())
 	return nextSibling();
@@ -688,7 +702,7 @@ NodeImpl *NodeImpl::traverseNextNode(NodeImpl *stayWithin) const
 	const NodeImpl *n = this;
 	while (n && !n->nextSibling() && (!stayWithin || n->parentNode() != stayWithin))
 	    n = n->parentNode();
-	if (n && (!stayWithin || n->parentNode() != stayWithin))
+	if (n)
 	    return n->nextSibling();
     }
     return 0;
@@ -824,6 +838,12 @@ NodeImpl::StyleChange NodeImpl::diff( khtml::RenderStyle *s1, khtml::RenderStyle
     return ch;
 }
 
+void NodeImpl::close()
+{
+    closeRenderer();
+    m_closed = true;
+}
+
 void NodeImpl::closeRenderer()
 {
     // It's important that we close the renderer, even if it hasn't been
@@ -905,6 +925,15 @@ bool NodeImpl::isReadOnly()
     return false;
 }
 
+RenderObject * NodeImpl::previousRenderer()
+{
+    for (NodeImpl *n = previousSibling(); n; n = n->previousSibling()) {
+        if (n->renderer())
+            return n->renderer();
+    }
+    return 0;
+}
+
 RenderObject * NodeImpl::nextRenderer()
 {
     for (NodeImpl *n = nextSibling(); n; n = n->nextSibling()) {
@@ -914,6 +943,48 @@ RenderObject * NodeImpl::nextRenderer()
     return 0;
 }
 
+void NodeImpl::createRendererIfNeeded()
+{
+#ifdef APPLE_CHANGES
+    if (!getDocument()->shouldCreateRenderers())
+        return;
+#endif
+
+    assert(!attached());
+    assert(!m_render);
+
+    NodeImpl *parent = parentNode();
+    assert(parent);
+
+    RenderObject *parentRenderer = parent->renderer();
+    if (parentRenderer && parentRenderer->childAllowed()) {
+        RenderStyle *style = styleForRenderer(parentRenderer);
+        style->ref();
+        if (rendererIsNeeded(style)) {
+            m_render = createRenderer(getDocument()->renderArena(), style);
+            m_render->setStyle(style);
+            parentRenderer->addChild(m_render, nextRenderer());
+        }
+        style->deref();
+    }
+}
+
+RenderStyle *NodeImpl::styleForRenderer(RenderObject *parent)
+{
+    return parent->style();
+}
+
+bool NodeImpl::rendererIsNeeded(RenderStyle *style)
+{
+    return (getDocument()->documentElement() == this) || (style->display() != NONE);
+}
+
+RenderObject *NodeImpl::createRenderer(RenderArena* /*arena*/, RenderStyle* /*style*/)
+{
+    assert(false);
+    return 0;
+}
+
 bool NodeImpl::contentEditable() const
 {
     RenderObject *r = renderer();
@@ -1547,128 +1618,202 @@ void NodeBaseImpl::dispatchChildRemovalEvents( NodeImpl *child, int &exceptionco
 }
 
 // ---------------------------------------------------------------------------
-
-NodeImpl *NodeListImpl::item( unsigned long /*index*/ ) const
+NodeImpl *NodeListImpl::item( unsigned long index ) const
 {
-    return 0;
+    unsigned long requestIndex = index;
+
+    m_cache->updateNodeListInfo(m_refNode->getDocument());
+
+    NodeImpl* n;
+    bool      usedCache = false;
+    if (m_cache->current.node) {
+        //Compute distance from the requested index to the cache node
+        long cacheDist = QABS(long(index) - long(m_cache->position));
+
+        if (cacheDist < index) { //Closer to the cached position
+            usedCache = true;
+            if (index >= m_cache->position) { //Go ahead
+                unsigned long relIndex = index - m_cache->position;
+                n = recursiveItem(m_refNode, m_cache->current.node, relIndex);
+            } else { //Go backwards
+                unsigned long relIndex = m_cache->position - index;
+                n = recursiveItemBack(m_refNode, m_cache->current.node, relIndex);
+            }
+        }
+    }
+
+    if (!usedCache)
+        n = recursiveItem(m_refNode, m_refNode->firstChild(), index);
+
+    //We always update the cache state, to make starting iteration
+    //where it was left off easy.
+    m_cache->current.node  = n;
+    m_cache->position = requestIndex;
+    return n;
 }
 
 unsigned long NodeListImpl::length() const
 {
-    return 0;
+    m_cache->updateNodeListInfo(m_refNode->getDocument());
+    if (!m_cache->hasLength) {
+        m_cache->length    = recursiveLength( m_refNode );
+        m_cache->hasLength = true;
+    }
+    return m_cache->length;
 }
 
 unsigned long NodeListImpl::recursiveLength(NodeImpl *start) const
 {
     unsigned long len = 0;
     for(NodeImpl *n = start->firstChild(); n != 0; n = n->nextSibling()) {
-        if ( n->nodeType() == Node::ELEMENT_NODE ) {
-            if (elementMatches(n))
+        bool recurse = true;
+        if (nodeMatches(n, recurse))
                 len++;
+        if (recurse)
             len+= recursiveLength(n);
         }
-    }
 
     return len;
 }
 
-NodeImpl *NodeListImpl::recursiveItem ( NodeImpl *start, unsigned long &offset ) const
+NodeListImpl::NodeListImpl( NodeImpl *n, int type, CacheFactory* factory )
 {
-    for(NodeImpl *n = start->firstChild(); n != 0; n = n->nextSibling()) {
-        if ( n->nodeType() == Node::ELEMENT_NODE ) {
-            if (elementMatches(n))
-                if (!offset--)
-                    return n;
+    m_refNode = n;
+    m_refNode->ref();
 
-            NodeImpl *depthSearch= recursiveItem(n, offset);
-            if (depthSearch)
-                return depthSearch;
-        }
+    m_cache = m_refNode->getDocument()->acquireCachedNodeListInfo(
+                  factory ? factory : Cache::make,
+                  n, type );
+}
+
+NodeListImpl::~NodeListImpl()
+{
+    m_refNode->getDocument()->releaseCachedNodeListInfo(m_cache);
+    m_refNode->deref();
+}
+
+
+/**
+ Next item in the pre-order walk of tree from node, but not going outside 
+ absStart
+*/
+static NodeImpl* helperNext(NodeImpl* node, NodeImpl* absStart)
+{
+    //Walk up until we wind a sibling to go to.
+    while (!node->nextSibling() && node != absStart)
+        node = node->parentNode();
+
+    if (node != absStart)
+        return node->nextSibling();
+    else
+        return 0;
+}
+
+NodeImpl *NodeListImpl::recursiveItem ( NodeImpl* absStart, NodeImpl *start, unsigned long &offset ) const
+{
+    for(NodeImpl *n = start; n != 0; n = helperNext(n, absStart)) {
+        bool recurse = true;
+        if (nodeMatches(n, recurse))
+            if (!offset--)
+                return n;
+
+        NodeImpl *depthSearch = recurse ? recursiveItem(n, n->firstChild(), offset) : 0;
+        if (depthSearch)
+            return depthSearch;
     }
 
     return 0; // no matching node in this subtree
 }
 
-ChildNodeListImpl::ChildNodeListImpl( NodeImpl *n )
+
+NodeImpl *NodeListImpl::recursiveItemBack ( NodeImpl* absStart, NodeImpl *start, unsigned long &offset ) const
 {
-    refNode = n;
-    refNode->ref();
-}
+    //### it might be cleaner/faster to split nodeMatches and recursion
+    //filtering.
+    bool dummy   = true;
+    NodeImpl* n  = start;
 
-ChildNodeListImpl::~ChildNodeListImpl()
-{
-    refNode->deref();
-}
+    do {
+        bool recurse = true;
 
-unsigned long ChildNodeListImpl::length() const
-{
-    unsigned long len = 0;
-    NodeImpl *n;
-    for(n = refNode->firstChild(); n != 0; n = n->nextSibling())
-        len++;
+        //Check whether the current node matches.
+        if (nodeMatches(n, dummy))
+            if (!offset--)
+                return n;
 
-    return len;
-}
+        if (n->previousSibling()) {
+            //Move to the last node of this whole subtree that we should recurse into
+            n       = n->previousSibling();
+            recurse = true;
 
-NodeImpl *ChildNodeListImpl::item ( unsigned long index ) const
-{
-    unsigned int pos = 0;
-    NodeImpl *n = refNode->firstChild();
-
-    while( n != 0 && pos < index )
-    {
-        n = n->nextSibling();
-        pos++;
+            while (n->lastChild()) {
+                (void)nodeMatches(n, recurse);
+                if (!recurse)
+                   break; //Don't go there
+                n = n->lastChild();
+            }
+        } else {
+            //We're done with this whole subtree, so move up
+            n = n->parentNode();
+        }
     }
+    while (n && n != absStart);
 
-    return n;
+    return 0;
 }
 
-bool ChildNodeListImpl::elementMatches( NodeImpl* /*testNode*/ ) const
+
+NodeListImpl::Cache::~Cache()
+{}
+
+void NodeListImpl::Cache::clear(DocumentImpl* doc)
 {
+   hasLength = false;
+   current.node = 0;
+   version   = doc->domTreeVersion();
+}
+
+void NodeListImpl::Cache::updateNodeListInfo(DocumentImpl* doc)
+{
+    //If version doesn't match, clear
+    if (doc->domTreeVersion() != version)
+        clear(doc);
+}
+
+ChildNodeListImpl::ChildNodeListImpl( NodeImpl *n ): NodeListImpl(n, CHILD_NODES)
+{}
+
+bool ChildNodeListImpl::nodeMatches( NodeImpl* /*testNode*/, bool& doRecurse ) const
+{
+    doRecurse = false;
     return true;
 }
 
 TagNodeListImpl::TagNodeListImpl( NodeImpl *n, NodeImpl::Id id )
-  : m_refNode(n),
+  : NodeListImpl(n, UNCACHEABLE),
     m_id(id),
     m_namespaceAware(false)
 {
-    m_refNode->ref();
     // An id of 0 here means "*" (match all nodes)
     m_matchAllNames = (id == 0);
     m_matchAllNamespaces = false;
 }
 
 TagNodeListImpl::TagNodeListImpl( NodeImpl *n, const DOMString &namespaceURI, const DOMString &localName )
-  : m_refNode(n),
+  : NodeListImpl(n, UNCACHEABLE),
     m_id(0),
     m_namespaceURI(namespaceURI),
     m_localName(localName),
     m_namespaceAware(true)
 {
-    m_refNode->ref();
     m_matchAllNames = (localName == "*");
     m_matchAllNamespaces = (namespaceURI == "*");
 }
 
-TagNodeListImpl::~TagNodeListImpl()
-{
-    m_refNode->deref();
-}
 
-unsigned long TagNodeListImpl::length() const
-{
-    return recursiveLength( m_refNode );
-}
-
-NodeImpl *TagNodeListImpl::item ( unsigned long index ) const
-{
-    return recursiveItem( m_refNode, index );
-}
-
-bool TagNodeListImpl::elementMatches( NodeImpl *testNode ) const
+bool TagNodeListImpl::nodeMatches( NodeImpl *testNode, bool& /*doRecurse*/ ) const
 {
+    if ( testNode->nodeType() != Node::ELEMENT_NODE ) return false;
     if (m_namespaceAware)
 	return (m_matchAllNamespaces || testNode->namespaceURI() == m_namespaceURI) &&
 	       (m_matchAllNames || testNode->localName() == m_localName);
@@ -1682,29 +1827,13 @@ bool TagNodeListImpl::elementMatches( NodeImpl *testNode ) const
 }
 
 NameNodeListImpl::NameNodeListImpl(NodeImpl *n, const DOMString &t )
-  : nodeName(t)
-{
-    refNode= n;
-    refNode->ref();
-}
+  : NodeListImpl(n, UNCACHEABLE),
+    nodeName(t)
+{}
 
-NameNodeListImpl::~NameNodeListImpl()
-{
-    refNode->deref();
-}
-
-unsigned long NameNodeListImpl::length() const
-{
-    return recursiveLength( refNode );
-}
-
-NodeImpl *NameNodeListImpl::item ( unsigned long index ) const
-{
-    return recursiveItem( refNode, index );
-}
-
-bool NameNodeListImpl::elementMatches( NodeImpl *testNode ) const
+bool NameNodeListImpl::nodeMatches( NodeImpl *testNode, bool& /*doRecurse*/ ) const
 {
+    if ( testNode->nodeType() != Node::ELEMENT_NODE ) return false;
     return static_cast<ElementImpl *>(testNode)->getAttribute(ATTR_NAME) == nodeName;
 }
 
@@ -1713,9 +1842,10 @@ NamedTagNodeListImpl::NamedTagNodeListImpl( NodeImpl *n, NodeImpl::Id tagId, con
 {
 }
 
-bool NamedTagNodeListImpl::elementMatches( NodeImpl *testNode ) const
+bool NamedTagNodeListImpl::nodeMatches( NodeImpl *testNode, bool& doRecurse ) const
 {
-    return TagNodeListImpl::elementMatches( testNode )
+    if ( testNode->nodeType() != Node::ELEMENT_NODE ) return false;
+    return TagNodeListImpl::nodeMatches( testNode, doRecurse )
         && static_cast<ElementImpl *>(testNode)->getAttribute(ATTR_NAME) == nodeName;
 }
 
