@@ -131,7 +131,8 @@ void KIconCanvas::slotLoadFiles()
     int i;
     QStringList::ConstIterator it;
     uint emitProgress = 10; // so we will emit it once in the beginning
-    for (it=mFiles.begin(), i=0; it!=mFiles.end(); it++, i++)
+    QStringList::ConstIterator end(mFiles.end());
+    for (it=mFiles.begin(), i=0; it!=end; ++it, i++)
     {
 	// Calling kapp->processEvents() makes the iconview flicker like hell
 	// (it's being repainted once for every new item), so we don't do this.
@@ -271,7 +272,6 @@ void KIconDialog::init()
     top->addWidget(bgroup);
     connect(bgroup, SIGNAL(clicked(int)), SLOT(slotButtonClicked(int)));
     QGridLayout *grid = new QGridLayout(bgroup->layout(), 3, 2);
-    grid->addRowSpacing(0, 15);
     mpRb1 = new QRadioButton(i18n("S&ystem icons:"), bgroup);
     grid->addWidget(mpRb1, 1, 0);
     mpCombo = new QComboBox(bgroup);
