@@ -1,5 +1,5 @@
 /*
- * $Id: tcpslavebase.cpp,v 1.134 2004/06/27 10:44:42 binner Exp $
+ * $Id: tcpslavebase.cpp 465272 2005-09-29 09:47:40Z mueller $
  *
  * Copyright (C) 2000 Alex Zepeda <zipzippy@sonic.net
  * Copyright (C) 2001-2003 George Staikos <staikos@kde.org>
@@ -19,8 +19,8 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -44,6 +44,9 @@
 #include <ksslall.h>
 #include <ksslcertdlg.h>
 #include <kmessagebox.h>
+#ifndef Q_WS_WIN //temporary
+#include <kresolver.h>
+#endif
 
 #include <klocale.h>
 #include <dcopclient.h>
@@ -53,6 +56,7 @@
 #include <kapplication.h>
 
 #include <kprotocolmanager.h>
+#include <kde_file.h>
 
 #include "kio/tcpslavebase.h"
 
@@ -147,6 +151,7 @@ TCPSlaveBase::~TCPSlaveBase()
 
 ssize_t TCPSlaveBase::write(const void *data, ssize_t len)
 {
+#ifdef Q_OS_UNIX
     if ( (m_bIsSSL || d->usingTLS) && !d->useSSLTunneling )
     {
         if ( d->needSSLHandShake )
@@ -154,10 +159,14 @@ ssize_t TCPSlaveBase::write(const void *data, ssize_t len)
         return d->kssl->write(data, len);
     }
     return KSocks::self()->write(m_iSock, data, len);
+#else
+    return 0;
+#endif
 }
 
 ssize_t TCPSlaveBase::read(void *data, ssize_t len)
 {
+#ifdef Q_OS_UNIX
     if ( (m_bIsSSL || d->usingTLS) && !d->useSSLTunneling )
     {
         if ( d->needSSLHandShake )
@@ -165,6 +174,9 @@ ssize_t TCPSlaveBase::read(void *data, ssize_t len)
         return d->kssl->read(data, len);
     }
     return KSocks::self()->read(m_iSock, data, len);
+#else
+    return 0;
+#endif
 }
 
 
@@ -190,7 +202,7 @@ ssize_t TCPSlaveBase::readLine(char *data, ssize_t len)
 
   char tmpbuf[1024];   // 1kb temporary buffer for peeking
   *data = 0;
-  int clen = 0;
+  ssize_t clen = 0;
   char *buf = data;
   int rc = 0;
 
@@ -248,7 +260,11 @@ if ((m_bIsSSL || d->usingTLS) && !d->useSSLTunneling) {       // SSL CASE
   }
 } else {                                                      // NON SSL CASE
   while (clen < len-1) {
+#ifdef Q_OS_UNIX
     rc = KSocks::self()->read(m_iSock, buf, 1);
+#else
+    rc = 0;
+#endif
     if (rc <= 0) {
       // FIXME: this doesn't cover rc == 0 case
       return -1;
@@ -287,6 +303,7 @@ bool TCPSlaveBase::connectToHost( const QString &host,
                                   unsigned int _port,
                                   bool sendError )
 {
+#ifdef Q_OS_UNIX
     unsigned short int p;
     KExtendedSocket ks;
 
@@ -375,12 +392,15 @@ bool TCPSlaveBase::connectToHost( const QString &host,
     // Since we want to use stdio on the socket,
     // we must fdopen it to get a file pointer,
     // if it fails, close everything up
-    if ((fp = fdopen(m_iSock, "w+")) == 0) {
+    if ((fp = KDE_fdopen(m_iSock, "w+")) == 0) {
         closeDescriptor();
         return false;
     }
 
     return true;
+#else //!Q_OS_UNIX
+    return false;
+#endif //Q_OS_UNIX
 }
 
 void TCPSlaveBase::closeDescriptor()
@@ -620,8 +640,9 @@ KSSLCertificateHome::KSSLAuthAction aa;
      QDataStream arg(data, IO_WriteOnly);
      arg << ourHost;
      arg << certs;
+     arg << metaData("window-id").toInt();
      bool rc = d->dcc->call("kio_uiserver", "UIServer",
-                               "showSSLCertDialog(QString, QStringList)",
+                               "showSSLCertDialog(QString, QStringList,int)",
                                data, rettype, retval);
 
      if (rc && rettype == "KSSLCertDlgRet") {
@@ -650,65 +671,30 @@ KSSLCertificateHome::KSSLAuthAction aa;
   KSSLPKCS12 *pkcs = KSSLCertificateHome::getCertificateByName(certname);
   if (!pkcs && KSSLCertificateHome::hasCertificateByName(certname)) {           // We need the password
      KIO::AuthInfo ai;
-     bool showprompt = !checkCachedAuthentication(ai);
+     bool first = true;
      do {
-        QString pass;
-        QByteArray authdata, authval;
-        QCString rettype;
-        QDataStream qds(authdata, IO_WriteOnly);
         ai.prompt = i18n("Enter the certificate password:");
         ai.caption = i18n("SSL Certificate Password");
-        ai.setModified(true);
+        ai.url.setProtocol("kssl");
+        ai.url.setHost(certname);
         ai.username = certname;
         ai.keepPassword = true;
+
+        bool showprompt;
+        if (first)
+           showprompt = !checkCachedAuthentication(ai);
+        else
+           showprompt = true;
         if (showprompt) {
-           qds << ai;
-
-           if (!d->dcc) {
-              d->dcc = new DCOPClient;
-              d->dcc->attach();
-              if (!d->dcc->isApplicationRegistered("kio_uiserver")) {
-                 KApplication::startServiceByDesktopPath("kio_uiserver.desktop",
-                                                         QStringList() );
-             }
-           }
-
-           bool rc = d->dcc->call("kio_uiserver", "UIServer",
-                                   "openPassDlg(KIO::AuthInfo)",
-                                   authdata, rettype, authval);
-           if (!rc) {
-             break;
-           }
-           if (rettype != "QByteArray") {
-             continue;
-           }
-
-           QDataStream qdret(authval, IO_ReadOnly);
-           QByteArray authdecode;
-           qdret >> authdecode;
-           QDataStream qdtoo(authdecode, IO_ReadOnly);
-           qdtoo >> ai;
-           if (!ai.isModified()) {
-             break;
-           }
+           if (!openPassDlg(ai, first ? QString::null : 
+                   i18n("Unable to open the certificate. Try a new password?")))
+              break;
         }
-        pass = ai.password;
-        pkcs = KSSLCertificateHome::getCertificateByName(certname, pass);
 
-        if (!pkcs) {
-              int rc = messageBox(WarningYesNo, i18n("Unable to open the "
-                                                     "certificate. Try a "
-                                                     "new password?"),
-                                                i18n("SSL"));
-              if (rc == KMessageBox::No) {
-                break;
-              }
-              showprompt = true;
-        }
+        first = false;
+        pkcs = KSSLCertificateHome::getCertificateByName(certname, ai.password);
      } while (!pkcs);
-     if (pkcs) {
-       cacheAuthentication(ai);
-     }
+
   }
 
    // If we could open the certificate, let's send it
@@ -774,13 +760,26 @@ int TCPSlaveBase::verifyCertificate()
     KSSLCertificate::KSSLValidationList ksvl = pc.validateVerbose(KSSLCertificate::SSLServer);
 
    _IPmatchesCN = d->kssl->peerInfo().certMatchesAddress();
-   if (!_IPmatchesCN && !d->militantSSL) {  // force this if the user wants it
-      if (d->cc->getHostList(pc).contains(ourHost))
-         _IPmatchesCN = true;
+   if (!_IPmatchesCN) {
+#ifndef Q_WS_WIN //temporary
+      KNetwork::KResolverResults res = KNetwork::KResolver::resolve(d->kssl->peerInfo().peerHost(), "80", KNetwork::KResolver::CanonName);
+      if (!res.isEmpty()) {
+         QString old = d->kssl->peerInfo().peerHost();
+         d->kssl->peerInfo().setPeerHost(res[0].canonicalName());
+         _IPmatchesCN = d->kssl->peerInfo().certMatchesAddress();
+         if (!_IPmatchesCN) {
+            d->kssl->peerInfo().setPeerHost(old);
+         }
+      }
+#endif
+      if (!_IPmatchesCN && !d->militantSSL) { // force this if the user wants it
+         if (d->cc->getHostList(pc).contains(ourHost)) {
+            _IPmatchesCN = true;
+         }
+      }
    }
 
-   if (!_IPmatchesCN)
-   {
+   if (!_IPmatchesCN) {
       ksvl << KSSLCertificate::InvalidHost;
    }
 
@@ -799,6 +798,9 @@ int TCPSlaveBase::verifyCertificate()
     setMetaData("ssl_cipher_bits",
                   QString::number(d->kssl->connectionInfo().getCipherBits()));
     setMetaData("ssl_peer_ip", d->ip);
+    if (!d->realHost.isEmpty()) {
+       setMetaData("ssl_proxied", "true");
+    }
     
     QString errorStr;
     for(KSSLCertificate::KSSLValidationList::ConstIterator it = ksvl.begin();
@@ -900,8 +902,9 @@ int TCPSlaveBase::verifyCertificate()
                    QCString ignoretype;
                    QDataStream arg(data, IO_WriteOnly);
                    arg << theurl << mOutgoingMetaData;
+                   arg << metaData("window-id").toInt();
                         d->dcc->call("kio_uiserver", "UIServer",
-                                "showSSLInfoDialog(QString,KIO::MetaData)",
+                                "showSSLInfoDialog(QString,KIO::MetaData,int)",
                                 data, ignoretype, ignore);
                 }
              } while (result == KMessageBox::Yes);
@@ -1026,8 +1029,9 @@ int TCPSlaveBase::verifyCertificate()
                    QCString ignoretype;
                    QDataStream arg(data, IO_WriteOnly);
                    arg << theurl << mOutgoingMetaData;
+                   arg << metaData("window-id").toInt();
                         d->dcc->call("kio_uiserver", "UIServer",
-                                "showSSLInfoDialog(QString,KIO::MetaData)",
+                                "showSSLInfoDialog(QString,KIO::MetaData,int)",
                                 data, ignoretype, ignore);
                 }
           } while (result == KMessageBox::Yes);
@@ -1109,8 +1113,9 @@ int TCPSlaveBase::verifyCertificate()
           QCString ignoretype;
           QDataStream arg(data, IO_WriteOnly);
           arg << theurl << mOutgoingMetaData;
+          arg << metaData("window-id").toInt();
           d->dcc->call("kio_uiserver", "UIServer",
-                       "showSSLInfoDialog(QString,KIO::MetaData)",
+                       "showSSLInfoDialog(QString,KIO::MetaData,int)",
                        data, ignoretype, ignore);
       }
       } while (result != KMessageBox::No);
@@ -1153,11 +1158,15 @@ bool TCPSlaveBase::isConnectionValid()
     tv.tv_usec = 0;
     tv.tv_sec = 0;
     int retval;
+#ifdef Q_OS_UNIX
     do {
        retval = KSocks::self()->select(m_iSock+1, &rdfs, NULL, NULL, &tv);
        if (wasKilled())
           return false; // Beam us out of here
     } while ((retval == -1) && (errno == EAGAIN));
+#else
+    retval = -1;
+#endif
     // retval == -1 ==> Error
     // retval ==  0 ==> Connection Idle
     // retval >=  1 ==> Connection Active
@@ -1172,10 +1181,14 @@ bool TCPSlaveBase::isConnectionValid()
 
     // Connection is active, check if it has closed.
     char buffer[100];
+#ifdef Q_OS_UNIX
     do {
        retval = KSocks::self()->recv(m_iSock, buffer, 80, MSG_PEEK);
 
     } while ((retval == -1) && (errno == EAGAIN));
+#else
+    retval = -1;
+#endif
     //kdDebug(7029) << "TCPSlaveBase::isConnectionValid: recv returned: "
     //                 << retval << endl;
     if (retval <= 0)
@@ -1206,7 +1219,11 @@ bool TCPSlaveBase::waitForResponse( int t )
 
 reSelect:
   startTime = time(NULL);
+#ifdef Q_OS_UNIX
   rc = KSocks::self()->select(m_iSock+1, &rd, NULL, NULL, &timeout);
+#else
+  rc = -1;
+#endif
   if (wasKilled())
     return false; // We're dead.
 
