@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -201,7 +201,8 @@ Value NumberProtoFuncImp::call(ExecState *exec, Object &thisObj, const List &arg
   case ValueOf:
     result = Number(v.toNumber(exec));
     break;
-  case ToFixed: {
+  case ToFixed:
+  {
     // FIXME: firefox works for all values, not just 0..20.  This includes
     // NaN, infinity, undefined, etc.  This is just a hack to pass our regression
     // suite.
@@ -210,7 +211,7 @@ Value NumberProtoFuncImp::call(ExecState *exec, Object &thisObj, const List &arg
     double fd = fractionDigits.toNumber(exec);
     if (isNaN(fd)) {
       f = 0;
-    } else if (finite(fd)) {
+    } else if (!isInf(fd)) {
       f = int(fd);
     }
     if (f < 0 || f > 20) {
@@ -347,7 +348,8 @@ Value NumberProtoFuncImp::call(ExecState *exec, Object &thisObj, const List &arg
 
     return String(UString(buf));
   }
-  case ToPrecision: {
+  case ToPrecision:
+  {
     int e = 0;
     UString m;
 
@@ -363,7 +365,8 @@ Value NumberProtoFuncImp::call(ExecState *exec, Object &thisObj, const List &arg
     }
 
     if (p < 1 || p > 21) {
-      Object err = Error::create(exec,RangeError);
+      Object err = Error::create(exec, RangeError,
+				 "toPrecision() argument must be between 1 and 21");
       exec->setException(err);
       return err;
     }
@@ -472,7 +475,7 @@ bool NumberObjectImp::implementsConstruct() const
 // ECMA 15.7.1
 Object NumberObjectImp::construct(ExecState *exec, const List &args)
 {
-  ObjectImp *proto = exec->interpreter()->builtinNumberPrototype().imp();
+  ObjectImp *proto = exec->lexicalInterpreter()->builtinNumberPrototype().imp();
   Object obj(new NumberInstanceImp(proto));
 
   Number n;
