@@ -17,8 +17,8 @@
  *
  *  You should have received a copy of the GNU Library General Public License
  *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- *  Boston, MA 02111-1307, USA.
+ *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ *  Boston, MA 02110-1301, USA.
  *
  */
 
@@ -176,7 +176,7 @@ Value ObjectImp::get(ExecState *exec, const Identifier &propertyName) const
       return Value(proto);
   }
 
-  if (proto.isNull())
+  if (!proto.isValid())
     return Undefined();
 
   return proto.get(exec,propertyName);
@@ -192,7 +192,7 @@ Value ObjectImp::getPropertyByIndex(ExecState *exec,
 void ObjectImp::put(ExecState *exec, const Identifier &propertyName,
                      const Value &value, int attr)
 {
-  assert(!value.isNull());
+  assert(value.isValid());
 
   // non-standard netscape extension
   if (propertyName == specialPrototypePropertyName) {
@@ -256,7 +256,7 @@ bool ObjectImp::hasProperty(ExecState *exec, const Identifier &propertyName) con
 
   // Look in the prototype
   Object proto = Object::dynamicCast(prototype());
-  return !proto.isNull() && proto.hasProperty(exec,propertyName);
+  return proto.isValid() && proto.hasProperty(exec,propertyName);
 }
 
 bool ObjectImp::hasPropertyByIndex(ExecState *exec, unsigned propertyName) const
@@ -265,7 +265,7 @@ bool ObjectImp::hasPropertyByIndex(ExecState *exec, unsigned propertyName) const
 }
 
 // ECMA 8.6.2.5
-bool ObjectImp::deleteProperty(ExecState */*exec*/, const Identifier &propertyName)
+bool ObjectImp::deleteProperty(ExecState * /*exec*/, const Identifier &propertyName)
 {
   int attributes;
   ValueImp *v = _prop.get(propertyName, attributes);
@@ -298,7 +298,7 @@ Value ObjectImp::defaultValue(ExecState *exec, Type hint) const
 {
   if (hint != StringType && hint != NumberType) {
     /* Prefer String for Date objects */
-    if (_proto == exec->interpreter()->builtinDatePrototype().imp())
+    if (_proto == exec->lexicalInterpreter()->builtinDatePrototype().imp())
       hint = StringType;
     else
       hint = NumberType;
@@ -462,7 +462,7 @@ UString ObjectImp::toString(ExecState *exec) const
   return prim.toString(exec);
 }
 
-Object ObjectImp::toObject(ExecState */*exec*/) const
+Object ObjectImp::toObject(ExecState * /*exec*/) const
 {
   return Object(const_cast<ObjectImp*>(this));
 }
@@ -510,25 +510,25 @@ Object Error::create(ExecState *exec, ErrorType errtype, const char *message,
 
   switch (errtype) {
   case EvalError:
-    cons = exec->interpreter()->builtinEvalError();
+    cons = exec->lexicalInterpreter()->builtinEvalError();
     break;
   case RangeError:
-    cons = exec->interpreter()->builtinRangeError();
+    cons = exec->lexicalInterpreter()->builtinRangeError();
     break;
   case ReferenceError:
-    cons = exec->interpreter()->builtinReferenceError();
+    cons = exec->lexicalInterpreter()->builtinReferenceError();
     break;
   case SyntaxError:
-    cons = exec->interpreter()->builtinSyntaxError();
+    cons = exec->lexicalInterpreter()->builtinSyntaxError();
     break;
   case TypeError:
-    cons = exec->interpreter()->builtinTypeError();
+    cons = exec->lexicalInterpreter()->builtinTypeError();
     break;
   case URIError:
-    cons = exec->interpreter()->builtinURIError();
+    cons = exec->lexicalInterpreter()->builtinURIError();
     break;
   default:
-    cons = exec->interpreter()->builtinError();
+    cons = exec->lexicalInterpreter()->builtinError();
     break;
   }
 
