@@ -14,8 +14,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #include "downloaddialog.h"
@@ -51,6 +51,10 @@ struct DownloadDialog::Private
     QString m_providerlist;
     QWidget *m_page;
     KListView *m_lvtmp_r, *m_lvtmp_d, *m_lvtmp_l;
+    QPtrList<Entry> m_installlist;
+    QMap<KIO::Job*, Provider*> m_variantjobs;
+    QMap<KIO::Job*, QStringList> m_variants;
+    QMap<Provider*, Provider*> m_newproviders;
 };
 
 class NumSortListViewItem : public KListViewItem
@@ -165,7 +169,8 @@ void DownloadDialog::load(QString providerList)
 void DownloadDialog::clear()
 {
   QMap<QWidget*, QValueList<KListView*>* >::Iterator it;
-  for(it = m_map.begin(); it != m_map.end(); it++)
+  QMap<QWidget*, QValueList<KListView*>* >::Iterator end(m_map.end());
+  for(it = m_map.begin(); it != end; ++it)
   {
     QValueList<KListView*> *v = it.data();
     kdDebug() << "clear listviews in " << v << endl;
@@ -221,10 +226,21 @@ void DownloadDialog::addProvider(Provider *p)
   kdDebug() << "addProvider()/begin" << endl;
 
   ret = true;
-  if(!p->icon().isValid()) ret = false;
-  else ret = KIO::NetAccess::download(p->icon(), tmp, this);
-  if(ret) pix = QPixmap(tmp);
-  else pix = KGlobal::iconLoader()->loadIcon("knewstuff", KIcon::Panel);
+  if(p->icon().path().isEmpty()) ret = false;
+  else
+  {
+    if(!p->icon().protocol().isEmpty())
+    {
+      ret = KIO::NetAccess::download(p->icon(), tmp, this);
+      if(ret) pix = QPixmap(tmp);
+    }
+    else
+    {
+      pix = KGlobal::iconLoader()->loadIcon(p->icon().path(), KIcon::Panel);
+      ret = true;
+    }
+  }
+  if(!ret) pix = KGlobal::iconLoader()->loadIcon("knewstuff", KIcon::Panel);
   frame = addPage(p->name(), p->name(), pix);
   m_frame = frame;
 
@@ -262,9 +278,9 @@ void DownloadDialog::addProvider(Provider *p)
   d->m_lvtmp_l->addColumn(i18n("Release Date"));
   d->m_lvtmp_l->setSorting(2, false);
 
-  connect(d->m_lvtmp_r, SIGNAL(selectionChanged()), SLOT(slotSelected()));
-  connect(d->m_lvtmp_d, SIGNAL(selectionChanged()), SLOT(slotSelected()));
-  connect(d->m_lvtmp_l, SIGNAL(selectionChanged()), SLOT(slotSelected()));
+  connect(d->m_lvtmp_r, SIGNAL(clicked(QListViewItem*)), SLOT(slotSelected()));
+  connect(d->m_lvtmp_d, SIGNAL(clicked(QListViewItem*)), SLOT(slotSelected()));
+  connect(d->m_lvtmp_l, SIGNAL(clicked(QListViewItem*)), SLOT(slotSelected()));
 
   rt = new QTextBrowser(frame);
   rt->setMinimumWidth(150);
@@ -281,6 +297,8 @@ void DownloadDialog::addProvider(Provider *p)
   vbox->add(de);
   vbox->add(in);
 
+  connect(rt, SIGNAL(linkClicked(const QString&)), SLOT(slotEmail(const QString&)));
+
   connect(in, SIGNAL(clicked()), SLOT(slotInstall()));
   connect(de, SIGNAL(clicked()), SLOT(slotDetails()));
 
@@ -334,18 +352,32 @@ void DownloadDialog::slotResult(KIO::Job *job)
         if((!m_filter.isEmpty()) && (entry->type() != m_filter)) continue;
       }
 
-      if((!m_filter.isEmpty()) && (m_jobs[job]))
+      /*if((!m_filter.isEmpty()) && (m_jobs[job]))
       {
         Provider *p = m_jobs[job];
-        addProvider(p);
-        slotPage(m_frame);
-        m_jobs[job] = 0;
+        if(d->m_newproviders[p])
+        {
+          addProvider(p);
+          slotPage(m_frame);
+          d->m_newproviders[p] = 0;
+        }
+      }*/
+      if((!m_filter.isEmpty()) && (d->m_variantjobs[job]))
+      {
+        Provider *p = d->m_variantjobs[job];
+        if(d->m_newproviders[p])
+        {
+          addProvider(p);
+          slotPage(m_frame);
+          d->m_newproviders[p] = 0;
+        }
       }
-      addEntry(entry);
+
+      /*if(m_jobs[job]) addEntry(entry);
+      else*/
+      if(d->m_variantjobs[job]) addEntry(entry, d->m_variants[job]);
     }
   }
-
-  m_data[job] = "";
 }
 
 int DownloadDialog::installStatus(Entry *entry)
@@ -354,8 +386,10 @@ int DownloadDialog::installStatus(Entry *entry)
   QString datestring;
   int installed;
 
+  QString lang = KGlobal::locale()->language();
+
   kapp->config()->setGroup("KNewStuffStatus");
-  datestring = kapp->config()->readEntry(entry->name());
+  datestring = kapp->config()->readEntry(entry->name(lang));
   if(datestring.isNull()) installed = 0;
   else
   {
@@ -368,41 +402,54 @@ int DownloadDialog::installStatus(Entry *entry)
   return installed;
 }
 
-void DownloadDialog::addEntry(Entry *entry)
+void DownloadDialog::addEntry(Entry *entry, const QStringList& variants)
 {
   QPixmap pix;
   int installed;
 
-  /*if(m_engine)
-  {
-    if(m_map.count() == 0)
-    {
-      m_frame = addPage(i18n("Welcome"), i18n("Welcome"), QPixmap(""));
-      Provider *p = new Provider();
-      p->setName(i18n("Generic"));
-      addProvider(p);
-      slotPage(m_frame);
-    }
-  }*/
   installed = installStatus(entry);
 
   if(installed > 0) pix = KGlobal::iconLoader()->loadIcon("ok", KIcon::Small);
   else if(installed < 0) pix = KGlobal::iconLoader()->loadIcon("history", KIcon::Small);
   else pix = QPixmap();
 
-  KListViewItem *tmp_r = new NumSortListViewItem(lv_r,
-    entry->name(), entry->version(), QString("%1").arg(entry->rating()));
-  KListViewItem *tmp_d = new NumSortListViewItem(lv_d,
-    entry->name(), entry->version(), QString("%1").arg(entry->downloads()));
-  KListViewItem *tmp_l = new DateSortListViewItem(lv_l,
-    entry->name(), entry->version(), KGlobal::locale()->formatDate(entry->releaseDate()));
+  QString lang = KGlobal::locale()->language();
 
-  tmp_r->setPixmap(0, pix);
-  tmp_d->setPixmap(0, pix);
-  tmp_l->setPixmap(0, pix);
+  if(variants.contains("score"))
+  {
+    KListViewItem *tmp_r = new NumSortListViewItem(lv_r,
+      entry->name(lang), entry->version(), QString("%1").arg(entry->rating()));
+    tmp_r->setPixmap(0, pix);
+  }
+  if(variants.contains("downloads"))
+  {
+    KListViewItem *tmp_d = new NumSortListViewItem(lv_d,
+      entry->name(lang), entry->version(), QString("%1").arg(entry->downloads()));
+    tmp_d->setPixmap(0, pix);
+  }
+  if(variants.contains("latest"))
+  {
+    KListViewItem *tmp_l = new DateSortListViewItem(lv_l,
+      entry->name(lang), entry->version(), KGlobal::locale()->formatDate(entry->releaseDate()));
+    tmp_l->setPixmap(0, pix);
+  }
 
   m_entries.append(entry);
 
+  kdDebug() << "added entry " << entry->name() << " for variants " << variants << endl;
+}
+
+void DownloadDialog::addEntry(Entry *entry)
+{
+  QStringList variants;
+
+  variants << "score";
+  variants << "downloads";
+  variants << "latest";
+
+  addEntry(entry, variants);
+
+  // not used anymore due to variants (but still used by engine)
   kdDebug() << "added entry " << entry->name() << endl;
 }
 
@@ -430,7 +477,7 @@ void DownloadDialog::slotDetails()
     "Downloads: %7\n"
     "Release date: %8\n"
     "Summary: %9\n"
-    ).arg(e->name()
+    ).arg(e->name(lang)
     ).arg(e->author()
     ).arg(e->license()
     ).arg(e->version()
@@ -452,13 +499,25 @@ void DownloadDialog::slotDetails()
   KMessageBox::information(this, info, i18n("Details"));
 }
 
+QListViewItem *DownloadDialog::currentEntryItem()
+{
+  if((m_curtab == 0) && (lv_r->selectedItem())) return lv_r->selectedItem();
+  if((m_curtab == 1) && (lv_d->selectedItem())) return lv_d->selectedItem();
+  if((m_curtab == 2) && (lv_l->selectedItem())) return lv_l->selectedItem();
+
+  return 0;
+}
+
 void DownloadDialog::slotInstall()
 {
   Entry *e = getEntry();
   if(!e) return;
+
   d->m_lvtmp_r->setEnabled( false );
   d->m_lvtmp_l->setEnabled( false );
   d->m_lvtmp_d->setEnabled( false );
+  m_entryitem = currentEntryItem();
+  m_entryname = m_entryitem->text(0);
 
   kdDebug() << "download entry now" << endl;
 
@@ -486,17 +545,30 @@ void DownloadDialog::install(Entry *e)
   kapp->config()->sync();
 
   QPixmap pix = KGlobal::iconLoader()->loadIcon("ok", KIcon::Small);
-  m_entryitem = lv_r->findItem(m_entryname, 0);
-  if(m_entryitem) m_entryitem->setPixmap(0, pix);
-  m_entryitem = lv_d->findItem(m_entryname, 0);
-  if(m_entryitem) m_entryitem->setPixmap(0, pix);
-  m_entryitem = lv_l->findItem(m_entryname, 0);
-  if(m_entryitem) m_entryitem->setPixmap(0, pix);
 
+  QString lang = KGlobal::locale()->language();
+  
+  if(m_entryitem)
+  {
+    m_entryitem->setPixmap(0, pix);
 
-  QPushButton *in;
-  in = *(m_buttons[d->m_page]->at(0));
-  if(in) in->setEnabled(false);
+    QListViewItem *item;
+    item = lv_r->findItem(e->name(lang), 0);
+    if(item) item->setPixmap(0, pix);
+    item = lv_d->findItem(e->name(lang), 0);
+    if(item) item->setPixmap(0, pix);
+    item = lv_l->findItem(e->name(lang), 0);
+    if(item) item->setPixmap(0, pix);
+  }
+
+  if(currentEntryItem() == m_entryitem)
+  {
+    QPushButton *in;
+    in = *(m_buttons[d->m_page]->at(0));
+    if(in) in->setEnabled(false);
+  }
+
+  d->m_installlist.append(e);
 }
 
 void DownloadDialog::slotInstalled(KIO::Job *job)
@@ -530,7 +602,28 @@ void DownloadDialog::slotInstalled(KIO::Job *job)
 void DownloadDialog::slotTab(int tab)
 {
   kdDebug() << "switch tab to: " << tab << endl;
+
+  Entry *eold = getEntry();
   m_curtab = tab;
+  Entry *e = getEntry();
+
+  if(e == eold) return;
+
+  if(e)
+  {
+    slotSelected();
+  }
+  else
+  {
+    QPushButton *de, *in;
+    in = *(m_buttons[d->m_page]->at(0));
+    de = *(m_buttons[d->m_page]->at(1));
+
+    if(in) in->setEnabled(false);
+    if(de) de->setEnabled(false);
+
+    m_rt->clear();
+  }
 }
 
 void DownloadDialog::slotSelected()
@@ -539,21 +632,65 @@ void DownloadDialog::slotSelected()
   bool enabled;
   Entry *e = getEntry();
   QString lang = KGlobal::locale()->language();
+  bool ret;
 
   if(e)
   {
+    QString lang = KGlobal::locale()->language();
+
+    QListViewItem *item;
+    if(m_curtab != 0)
+    {
+      lv_r->clearSelection();
+      item = lv_r->findItem(e->name(lang), 0);
+      if(item) lv_r->setSelected(item, true);
+    }
+    if(m_curtab != 1)
+    {
+      lv_d->clearSelection();
+      item = lv_d->findItem(e->name(lang), 0);
+      if(item) lv_d->setSelected(item, true);
+    }
+    if(m_curtab != 2)
+    {
+      lv_l->clearSelection();
+      item = lv_l->findItem(e->name(lang), 0);
+      if(item) lv_l->setSelected(item, true);
+    }
+
     if(!e->preview(lang).isValid())
     {
-      m_rt->setText(QString("<b>%1</b><br>%2<br>%3<br><br><i>%4</i><br>(%5)").arg(
-        e->name()).arg(e->author()).arg(KGlobal::locale()->formatDate(e->releaseDate())).arg(e->summary(lang)).arg(e->license()));
+      ret = 0;
     }
     else
     {
-      KIO::NetAccess::download(e->preview(lang), tmp, this);
-      m_rt->setText(QString("<b>%1</b><br>%2<br>%3<br><br><img src='%4'><br><i>%5</i><br>(%6)").arg(
-        e->name()).arg(e->author()).arg(KGlobal::locale()->formatDate(e->releaseDate())).arg(tmp).arg(e->summary(lang)).arg(e->license()));
+      ret = KIO::NetAccess::download(e->preview(lang), tmp, this);
     }
 
+    QString desc = QString("<b>%1</b><br>").arg(e->name(lang));
+    if(!e->authorEmail().isNull())
+    {
+      desc += QString("<a href='mailto:" + e->authorEmail() + "'>" + e->author() + "</a>");
+    }
+    else
+    {
+      desc += QString("%1").arg(e->author());
+    }
+    desc += QString("<br>%1").arg(KGlobal::locale()->formatDate(e->releaseDate()));
+    desc += QString("<br><br>");
+    if(ret)
+    {
+      desc += QString("<img src='%1'>").arg(tmp);
+    }
+    else
+    {
+      desc += i18n("Preview not available.");
+    }
+    desc += QString("<br><i>%1</i>").arg(e->summary(lang));
+    desc += QString("<br>(%1)").arg(e->license());
+
+    m_rt->setText(desc);
+
     if(installStatus(e) == 1) enabled = false;
     else enabled = true;
 
@@ -565,19 +702,27 @@ void DownloadDialog::slotSelected()
   }
 }
 
+void DownloadDialog::slotEmail(const QString& link)
+{
+  kdDebug() << "EMAIL: " << link << endl;
+  kapp->invokeMailer(link);
+  slotSelected(); // QTextBrowser oddity workaround as it cannot handle mailto: URLs
+}
+
 Entry *DownloadDialog::getEntry()
 {
-  if(m_curtab == 0) m_entryitem = lv_r->currentItem();
-  else if(m_curtab == 1) m_entryitem = lv_d->currentItem();
-  else if(m_curtab == 2) m_entryitem = lv_l->currentItem();
-  else return 0;
+  QListViewItem *entryItem = currentEntryItem();
 
-  m_entryname = m_entryitem->text(0);
+  if(!entryItem)
+    return 0;
+
+  QString entryName = entryItem->text(0);
+
+  QString lang = KGlobal::locale()->language();
 
   for(Entry *e = m_entries.first(); e; e = m_entries.next())
-  {
-    if(e->name() == m_entryname) return e;
-  }
+    if(e->name(lang) == entryName)
+      return e;
 
   return 0;
 }
@@ -615,9 +760,70 @@ void DownloadDialog::slotPage(QWidget *w)
 
 void DownloadDialog::loadProvider(Provider *p)
 {
+  QMap<KIO::Job*, Provider*>::Iterator it;
+
+  for(it = d->m_variantjobs.begin(); it != d->m_variantjobs.end(); it++)
+  {
+    if(it.data() == p)
+    {
+      kdDebug() << "-- found provider data in cache" << endl;
+      slotResult(it.key());
+      return;
+    }
+  }
+
+  QStringList variants;
+  variants << "score";
+  variants << "downloads";
+  variants << "latest";
+
+  // Optimise URLs so each unique URL only gets fetched once
+
+  QMap<QString, QStringList> urls;
+
+  for(QStringList::Iterator it = variants.begin(); it != variants.end(); it++)
+  {
+    QString url = p->downloadUrlVariant((*it)).url();
+    if(!urls.contains(url))
+    {
+      urls[url] = QStringList();
+    }
+    urls[url] << (*it);
+
+    it = variants.remove(it);
+  }
+
+  // Now fetch the URLs while keeping the variant list for each attached
+
+  for(QMap<QString, QStringList>::Iterator it = urls.begin(); it != urls.end(); it++)
+  {
+    QString url = it.key();
+    QStringList urlvariants = it.data();
+
+    KIO::TransferJob *variantjob = KIO::get(url);
+    d->m_newproviders[p] = p;
+    d->m_variantjobs[variantjob] = p;
+    d->m_variants[variantjob] = urlvariants;
+    m_data[variantjob] = "";
+
+    connect(variantjob, SIGNAL(result(KIO::Job*)), SLOT(slotResult(KIO::Job*)));
+    connect(variantjob, SIGNAL(data(KIO::Job*, const QByteArray&)),
+      SLOT(slotData(KIO::Job*, const QByteArray&)));
+  }
+
+  if(variants.count() == 0) return;
+
+  // If not all variants are given, use default URL for those
+
+  kdDebug() << "-- reached old downloadurl section; variants left: " << variants.count() << endl;
+
   KIO::TransferJob *job = KIO::get(p->downloadUrl());
 
-  m_jobs[job] = p;
+  d->m_newproviders[p] = p;
+  d->m_variantjobs[job] = p;
+  d->m_variants[job] = variants;
+  //m_jobs[job] = p; // not used anymore due to variants
+  m_data[job] = "";
 
   connect(job, SIGNAL(result(KIO::Job*)), SLOT(slotResult(KIO::Job*)));
   connect(job, SIGNAL(data(KIO::Job*, const QByteArray&)),
@@ -656,3 +862,7 @@ void DownloadDialog::slotFinish()
   //updateBackground();
 }
 
+QPtrList<Entry> DownloadDialog::installedEntries()
+{
+  return d->m_installlist;
+}
