@@ -17,10 +17,10 @@
  
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 
-    $Id: player.cc,v 1.43 2002/12/07 16:39:52 waba Exp $
+    $Id: player.cc 465272 2005-09-29 09:47:40Z mueller $
 
     Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>
 
